"""Tests for __main__.py: argparse dispatch and subcommand functions."""

from __future__ import annotations

import argparse
import sqlite3
from pathlib import Path
from unittest.mock import MagicMock, patch

import pytest

from anteroom.db import _SCHEMA, ThreadSafeConnection

# ---------------------------------------------------------------------------
# Shared helpers
# ---------------------------------------------------------------------------


def _make_config(port: int = 8080, host: str = "127.0.0.1") -> MagicMock:
    config = MagicMock()
    config.app.port = port
    config.app.host = host
    config.app.data_dir = Path("/tmp/anteroom-test")
    config.app.tls = False
    config.ai.base_url = "http://localhost:1234/v1"
    config.ai.model = "test-model"
    config.ai.verify_ssl = True
    config.ai.temperature = None
    config.ai.top_p = None
    config.ai.seed = None
    config.mcp_servers = []
    config.safety.approval_mode = "ask_for_writes"
    config.safety.allowed_tools = []
    config.safety.read_only = False
    config.compliance.rules = []
    config.audit.log_path = None
    config.audit.retention_days = 30
    config.identity = MagicMock()
    config.identity.private_key = "fake-private-key"
    config.storage.retention_days = 0
    config.storage.purge_attachments = True
    config.pack_sources = []
    return config


@pytest.fixture()
def db() -> ThreadSafeConnection:
    conn = sqlite3.connect(":memory:", check_same_thread=False)
    conn.row_factory = sqlite3.Row
    conn.execute("PRAGMA foreign_keys=ON")
    conn.executescript(_SCHEMA)
    conn.commit()
    return ThreadSafeConnection(conn)


# ---------------------------------------------------------------------------
# _run_init
# ---------------------------------------------------------------------------


class TestRunInit:
    def test_run_init_calls_wizard(self) -> None:
        from anteroom.__main__ import _run_init

        with patch("anteroom.cli.setup.run_init_wizard") as mock_wizard:
            _run_init(force=False, team_config=None)

        mock_wizard.assert_called_once_with(force=False, team_config_path=None)

    def test_run_init_passes_force_flag(self) -> None:
        from anteroom.__main__ import _run_init

        with patch("anteroom.cli.setup.run_init_wizard") as mock_wizard:
            _run_init(force=True, team_config=None)

        mock_wizard.assert_called_once_with(force=True, team_config_path=None)

    def test_run_init_passes_team_config(self) -> None:
        from anteroom.__main__ import _run_init

        with patch("anteroom.cli.setup.run_init_wizard") as mock_wizard:
            _run_init(force=False, team_config="/tmp/team.yaml")

        mock_wizard.assert_called_once_with(force=False, team_config_path="/tmp/team.yaml")

    def test_main_dispatches_init(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._run_init") as mock_init,
            patch("sys.argv", ["aroom", "init"]),
        ):
            main()

        mock_init.assert_called_once_with(force=False, team_config=None)

    def test_main_init_force_flag(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._run_init") as mock_init,
            patch("sys.argv", ["aroom", "init", "--force"]),
        ):
            main()

        mock_init.assert_called_once_with(force=True, team_config=None)

    def test_main_init_team_config_flag(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._run_init") as mock_init,
            patch("sys.argv", ["aroom", "init", "--team-config", "/tmp/team.yaml"]),
        ):
            main()

        mock_init.assert_called_once_with(force=False, team_config="/tmp/team.yaml")


# ---------------------------------------------------------------------------
# _load_config_or_exit
# ---------------------------------------------------------------------------


class TestLoadConfigOrExit:
    def test_exits_when_config_missing_and_wizard_fails(
        self, tmp_path: Path, capsys: pytest.CaptureFixture[str]
    ) -> None:
        from anteroom.__main__ import _load_config_or_exit

        nonexistent = tmp_path / "no_config.yaml"

        with (
            patch("anteroom.__main__._get_config_path", return_value=nonexistent),
            patch("anteroom.cli.setup.run_init_wizard", return_value=False),
            pytest.raises(SystemExit) as exc_info,
        ):
            _load_config_or_exit()

        assert exc_info.value.code == 1

    def test_exits_on_value_error(self, tmp_path: Path, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _load_config_or_exit

        config_file = tmp_path / "config.yaml"
        config_file.write_text("ai:\n  base_url: http://localhost\n  api_key: test\n")

        with (
            patch("anteroom.__main__._get_config_path", return_value=config_file),
            patch("anteroom.__main__.load_config", side_effect=ValueError("Bad config")),
            pytest.raises(SystemExit) as exc_info,
        ):
            _load_config_or_exit()

        assert exc_info.value.code == 1
        captured = capsys.readouterr()
        assert "Configuration error" in captured.err

    def test_exits_on_compliance_failure(self, tmp_path: Path) -> None:
        from anteroom.__main__ import _load_config_or_exit

        config_file = tmp_path / "config.yaml"
        config_file.write_text("ai:\n  base_url: http://localhost\n  api_key: test\n")

        mock_config = _make_config()
        mock_result = MagicMock()
        mock_result.is_compliant = False
        mock_result.format_report.return_value = "Rules violated"

        with (
            patch("anteroom.__main__._get_config_path", return_value=config_file),
            patch("anteroom.__main__.load_config", return_value=(mock_config, [])),
            patch("anteroom.services.compliance.validate_compliance", return_value=mock_result),
            pytest.raises(SystemExit) as exc_info,
        ):
            _load_config_or_exit()

        assert exc_info.value.code == 1

    def test_returns_config_on_success(self, tmp_path: Path) -> None:
        from anteroom.__main__ import _load_config_or_exit

        config_file = tmp_path / "config.yaml"
        config_file.write_text("ai:\n  base_url: http://localhost\n  api_key: test\n")

        mock_config = _make_config()
        mock_result = MagicMock()
        mock_result.is_compliant = True

        with (
            patch("anteroom.__main__._get_config_path", return_value=config_file),
            patch("anteroom.__main__.load_config", return_value=(mock_config, ["field1"])),
            patch("anteroom.services.compliance.validate_compliance", return_value=mock_result),
        ):
            path, config, enforced = _load_config_or_exit()

        assert path == config_file
        assert config is mock_config
        assert enforced == ["field1"]


# ---------------------------------------------------------------------------
# _run_config
# ---------------------------------------------------------------------------


class TestRunConfig:
    def test_main_config_validate_dispatches(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._run_config_validate") as mock_validate,
            patch("sys.argv", ["aroom", "config", "validate"]),
        ):
            main()

        mock_validate.assert_called_once()

    def test_main_config_no_subcommand_opens_editor(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.cli.setup.run_config_editor") as mock_editor,
            patch("sys.argv", ["aroom", "config"]),
        ):
            main()

        mock_editor.assert_called_once()

    def test_run_config_validate_exits_0_when_compliant(self, tmp_path: Path) -> None:
        from anteroom.__main__ import _run_config_validate

        mock_config = _make_config()
        mock_config.compliance.rules = []
        mock_result = MagicMock()
        mock_result.is_compliant = True

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(tmp_path / "config.yaml", mock_config, [])),
            patch("anteroom.services.compliance.validate_compliance", return_value=mock_result),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_config_validate()

        assert exc_info.value.code == 0

    def test_run_config_validate_exits_1_when_not_compliant(self, tmp_path: Path) -> None:
        from anteroom.__main__ import _run_config_validate

        mock_config = _make_config()
        mock_config.compliance.rules = [MagicMock()]
        mock_result = MagicMock()
        mock_result.is_compliant = False
        mock_result.format_report.return_value = "Violations found"

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(tmp_path / "config.yaml", mock_config, [])),
            patch("anteroom.services.compliance.validate_compliance", return_value=mock_result),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_config_validate()

        assert exc_info.value.code == 1

    def test_run_config_validate_no_rules_message(self, tmp_path: Path, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_config_validate

        mock_config = _make_config()
        mock_config.compliance.rules = []
        mock_result = MagicMock()
        mock_result.is_compliant = True

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(tmp_path / "config.yaml", mock_config, [])),
            patch("anteroom.services.compliance.validate_compliance", return_value=mock_result),
            pytest.raises(SystemExit),
        ):
            _run_config_validate()

        captured = capsys.readouterr()
        assert "no rules defined" in captured.out

    def test_run_config_validate_shows_rule_count(self, tmp_path: Path, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_config_validate

        mock_config = _make_config()
        mock_config.compliance.rules = [MagicMock(), MagicMock()]
        mock_result = MagicMock()
        mock_result.is_compliant = True

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(tmp_path / "config.yaml", mock_config, [])),
            patch("anteroom.services.compliance.validate_compliance", return_value=mock_result),
            pytest.raises(SystemExit),
        ):
            _run_config_validate()

        captured = capsys.readouterr()
        assert "2 rule(s) passed" in captured.out


# ---------------------------------------------------------------------------
# _run_db
# ---------------------------------------------------------------------------


class TestRunDb:
    def test_main_dispatches_db(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._run_db") as mock_run_db,
            patch("sys.argv", ["aroom", "db", "list"]),
        ):
            main()

        mock_run_db.assert_called_once()

    def test_db_list_empty(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_db

        config_file = tmp_path / "config.yaml"
        config_file.write_text("{}")

        args = argparse.Namespace(db_action="list", name=None, path=None)

        with patch("anteroom.__main__._get_config_path", return_value=config_file):
            _run_db(args)

        captured = capsys.readouterr()
        assert "No shared databases" in captured.out

    def test_db_list_shows_databases(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_db

        config_file = tmp_path / "config.yaml"
        config_file.write_text("databases:\n  mydb:\n    path: /tmp/test.db\n")

        args = argparse.Namespace(db_action="list", name=None, path=None)

        with patch("anteroom.__main__._get_config_path", return_value=config_file):
            _run_db(args)

        captured = capsys.readouterr()
        assert "mydb" in captured.out
        assert "/tmp/test.db" in captured.out

    def test_db_list_shows_passphrase_auth(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_db

        config_file = tmp_path / "config.yaml"
        config_file.write_text("databases:\n  mydb:\n    path: /tmp/test.db\n    passphrase_hash: abc123\n")

        args = argparse.Namespace(db_action="list", name=None, path=None)

        with patch("anteroom.__main__._get_config_path", return_value=config_file):
            _run_db(args)

        captured = capsys.readouterr()
        assert "auth: yes" in captured.out

    def test_db_unknown_action_exits(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_db

        config_file = tmp_path / "config.yaml"
        config_file.write_text("{}")

        args = argparse.Namespace(db_action="unknown_action", name=None, path=None)

        with (
            patch("anteroom.__main__._get_config_path", return_value=config_file),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_db(args)

        assert exc_info.value.code == 1

    def test_db_connect_missing_name_exits(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_db

        config_file = tmp_path / "config.yaml"
        config_file.write_text("{}")

        args = argparse.Namespace(db_action="connect", name=None, path=None)

        with (
            patch("anteroom.__main__._get_config_path", return_value=config_file),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_db(args)

        assert exc_info.value.code == 1

    def test_db_connect_not_found_exits(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_db

        config_file = tmp_path / "config.yaml"
        config_file.write_text("databases:\n  other_db:\n    path: /tmp/other.db\n")

        args = argparse.Namespace(db_action="connect", name="nonexistent", path=None)

        with (
            patch("anteroom.__main__._get_config_path", return_value=config_file),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_db(args)

        assert exc_info.value.code == 1

    def test_db_connect_success_no_passphrase(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_db

        config_file = tmp_path / "config.yaml"
        config_file.write_text("databases:\n  mydb:\n    path: /tmp/test.db\n")

        args = argparse.Namespace(db_action="connect", name="mydb", path=None)

        with patch("anteroom.__main__._get_config_path", return_value=config_file):
            _run_db(args)

        captured = capsys.readouterr()
        assert "Connected to 'mydb'" in captured.out

    def test_db_purge_dispatches(self, tmp_path: Path) -> None:
        from anteroom.__main__ import _run_db

        config_file = tmp_path / "config.yaml"
        config_file.write_text("{}")

        args = argparse.Namespace(db_action="purge", name=None, path=None)

        with (
            patch("anteroom.__main__._get_config_path", return_value=config_file),
            patch("anteroom.__main__._run_db_purge") as mock_purge,
        ):
            _run_db(args)

        mock_purge.assert_called_once_with(args)

    def test_db_encrypt_dispatches(self, tmp_path: Path) -> None:
        from anteroom.__main__ import _run_db

        config_file = tmp_path / "config.yaml"
        config_file.write_text("{}")

        args = argparse.Namespace(db_action="encrypt", name=None, path=None)

        with (
            patch("anteroom.__main__._get_config_path", return_value=config_file),
            patch("anteroom.__main__._run_db_encrypt") as mock_encrypt,
        ):
            _run_db(args)

        mock_encrypt.assert_called_once_with(args)


# ---------------------------------------------------------------------------
# _run_db_purge
# ---------------------------------------------------------------------------


class TestRunDbPurge:
    def test_purge_requires_before_or_older_than(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_db_purge

        mock_config = _make_config()
        mock_config.storage.retention_days = 0

        args = argparse.Namespace(before=None, older_than=None, dry_run=False, yes=False)

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_db_purge(args)

        assert exc_info.value.code == 1

    def test_purge_uses_config_retention_when_no_arg(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_db_purge

        mock_config = _make_config()
        mock_config.storage.retention_days = 30
        mock_config.app.data_dir = Path("/tmp/anteroom-test")

        args = argparse.Namespace(before=None, older_than=None, dry_run=True, yes=True)

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            patch("anteroom.db.init_db") as mock_init_db,
            patch("anteroom.services.retention.purge_conversations_before", return_value=0),
            patch("anteroom.services.retention.purge_orphaned_attachments", return_value=0),
        ):
            mock_db = MagicMock()
            mock_init_db.return_value = mock_db
            _run_db_purge(args)

        captured = capsys.readouterr()
        assert "30 days" in captured.out or "retention" in captured.out

    def test_purge_invalid_before_date_exits(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_db_purge

        mock_config = _make_config()

        args = argparse.Namespace(before="not-a-date", older_than=None, dry_run=False, yes=False)

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_db_purge(args)

        assert exc_info.value.code == 1

    def test_purge_invalid_older_than_format_exits(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_db_purge

        mock_config = _make_config()

        args = argparse.Namespace(before=None, older_than="30", dry_run=False, yes=False)

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_db_purge(args)

        assert exc_info.value.code == 1
        captured = capsys.readouterr()
        assert "must end with 'd'" in captured.err

    def test_purge_invalid_older_than_number_exits(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_db_purge

        mock_config = _make_config()

        args = argparse.Namespace(before=None, older_than="xd", dry_run=False, yes=False)

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_db_purge(args)

        assert exc_info.value.code == 1

    def test_purge_dry_run_shows_preview(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_db_purge

        mock_config = _make_config()
        mock_config.app.data_dir = Path("/tmp/anteroom-test")

        args = argparse.Namespace(before="2024-01-01", older_than=None, dry_run=True, yes=False)

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            patch("anteroom.db.init_db") as mock_init_db,
            patch("anteroom.services.retention.purge_conversations_before", return_value=5),
            patch("anteroom.services.retention.purge_orphaned_attachments", return_value=2),
        ):
            mock_db = MagicMock()
            mock_init_db.return_value = mock_db
            _run_db_purge(args)

        captured = capsys.readouterr()
        assert "dry run" in captured.out
        assert "5" in captured.out

    def test_purge_nothing_to_purge_returns_early(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_db_purge

        mock_config = _make_config()
        mock_config.app.data_dir = Path("/tmp/anteroom-test")

        args = argparse.Namespace(before="2024-01-01", older_than=None, dry_run=False, yes=False)

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            patch("anteroom.db.init_db") as mock_init_db,
            patch("anteroom.services.retention.purge_conversations_before", return_value=0),
            patch("anteroom.services.retention.purge_orphaned_attachments", return_value=0),
        ):
            mock_db = MagicMock()
            mock_init_db.return_value = mock_db
            _run_db_purge(args)

        captured = capsys.readouterr()
        assert "Nothing to purge" in captured.out

    def test_purge_with_yes_flag_skips_confirm(self) -> None:
        from anteroom.__main__ import _run_db_purge

        mock_config = _make_config()
        mock_config.app.data_dir = Path("/tmp/anteroom-test")
        mock_config.storage.purge_attachments = True

        args = argparse.Namespace(before="2024-01-01", older_than=None, dry_run=False, yes=True)

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            patch("anteroom.db.init_db") as mock_init_db,
            patch("anteroom.services.retention.purge_conversations_before", return_value=3) as mock_purge,
            patch("anteroom.services.retention.purge_orphaned_attachments", return_value=1),
        ):
            mock_db = MagicMock()
            mock_init_db.return_value = mock_db
            _run_db_purge(args)

        # purge was called twice: once for dry_run preview, once for real
        assert mock_purge.call_count == 2


# ---------------------------------------------------------------------------
# _run_db_encrypt
# ---------------------------------------------------------------------------


class TestRunDbEncrypt:
    def test_encrypt_exits_when_sqlcipher_not_available(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_db_encrypt

        mock_config = _make_config()

        args = argparse.Namespace(yes=True)

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            patch("anteroom.services.encryption.is_sqlcipher_available", return_value=False),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_db_encrypt(args)

        assert exc_info.value.code == 1
        captured = capsys.readouterr()
        assert "sqlcipher3" in captured.err

    def test_encrypt_exits_when_no_identity_key(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_db_encrypt

        mock_config = _make_config()
        mock_config.identity.private_key = ""

        args = argparse.Namespace(yes=True)

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            patch("anteroom.services.encryption.is_sqlcipher_available", return_value=True),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_db_encrypt(args)

        assert exc_info.value.code == 1

    def test_encrypt_exits_when_db_not_found(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_db_encrypt

        mock_config = _make_config()
        mock_config.app.data_dir = tmp_path  # no chat.db here

        args = argparse.Namespace(yes=True)

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            patch("anteroom.services.encryption.is_sqlcipher_available", return_value=True),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_db_encrypt(args)

        assert exc_info.value.code == 1
        captured = capsys.readouterr()
        assert "database not found" in captured.err

    def test_encrypt_success(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_db_encrypt

        db_path = tmp_path / "chat.db"
        db_path.write_bytes(b"fake db content")

        mock_config = _make_config()
        mock_config.app.data_dir = tmp_path

        args = argparse.Namespace(yes=True)

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            patch("anteroom.services.encryption.is_sqlcipher_available", return_value=True),
            patch("anteroom.services.encryption.derive_db_key", return_value=b"key"),
            patch(
                "anteroom.services.encryption.migrate_plaintext_to_encrypted",
                return_value=tmp_path / "chat.db.bak-plaintext",
            ),
        ):
            _run_db_encrypt(args)

        captured = capsys.readouterr()
        assert "Encryption complete" in captured.out

    def test_encrypt_migration_error_exits(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_db_encrypt

        db_path = tmp_path / "chat.db"
        db_path.write_bytes(b"fake db content")

        mock_config = _make_config()
        mock_config.app.data_dir = tmp_path

        args = argparse.Namespace(yes=True)

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            patch("anteroom.services.encryption.is_sqlcipher_available", return_value=True),
            patch("anteroom.services.encryption.derive_db_key", return_value=b"key"),
            patch(
                "anteroom.services.encryption.migrate_plaintext_to_encrypted",
                side_effect=RuntimeError("migration failed"),
            ),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_db_encrypt(args)

        assert exc_info.value.code == 1


# ---------------------------------------------------------------------------
# _run_usage
# ---------------------------------------------------------------------------


class TestRunUsage:
    def test_run_usage_all_periods(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_usage

        mock_config = _make_config()
        mock_config.app.data_dir = Path("/tmp/anteroom-test")
        mock_config.cli.usage.week_days = 7
        mock_config.cli.usage.month_days = 30
        mock_config.cli.usage.model_costs = {}

        with (
            patch("anteroom.db.init_db") as mock_init_db,
            patch("anteroom.services.storage.get_usage_stats", return_value=[]),
        ):
            mock_db = MagicMock()
            mock_init_db.return_value = mock_db
            _run_usage(mock_config)

        captured = capsys.readouterr()
        assert "Token Usage" in captured.out

    def test_run_usage_single_period(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_usage

        mock_config = _make_config()
        mock_config.app.data_dir = Path("/tmp/anteroom-test")
        mock_config.cli.usage.week_days = 7
        mock_config.cli.usage.month_days = 30
        mock_config.cli.usage.model_costs = {}

        with (
            patch("anteroom.db.init_db") as mock_init_db,
            patch("anteroom.services.storage.get_usage_stats", return_value=[]),
        ):
            mock_db = MagicMock()
            mock_init_db.return_value = mock_db
            _run_usage(mock_config, period="day")

        captured = capsys.readouterr()
        assert "Token Usage" in captured.out

    def test_run_usage_json_output(self, capsys: pytest.CaptureFixture[str]) -> None:
        import json

        from anteroom.__main__ import _run_usage

        mock_config = _make_config()
        mock_config.app.data_dir = Path("/tmp/anteroom-test")
        mock_config.cli.usage.week_days = 7
        mock_config.cli.usage.month_days = 30
        mock_config.cli.usage.model_costs = {}

        with (
            patch("anteroom.db.init_db") as mock_init_db,
            patch("anteroom.services.storage.get_usage_stats", return_value=[]),
        ):
            mock_db = MagicMock()
            mock_init_db.return_value = mock_db
            _run_usage(mock_config, output_json=True)

        captured = capsys.readouterr()
        data = json.loads(captured.out)
        assert isinstance(data, dict)

    def test_run_usage_with_stats(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_usage

        mock_config = _make_config()
        mock_config.app.data_dir = Path("/tmp/anteroom-test")
        mock_config.cli.usage.week_days = 7
        mock_config.cli.usage.month_days = 30
        mock_config.cli.usage.model_costs = {"gpt-4": {"input": 0.03, "output": 0.06}}

        stats = [
            {
                "model": "gpt-4",
                "prompt_tokens": 1000,
                "completion_tokens": 500,
                "total_tokens": 1500,
                "message_count": 5,
            }
        ]

        with (
            patch("anteroom.db.init_db") as mock_init_db,
            patch("anteroom.services.storage.get_usage_stats", return_value=stats),
        ):
            mock_db = MagicMock()
            mock_init_db.return_value = mock_db
            _run_usage(mock_config, period="all")

        captured = capsys.readouterr()
        assert "1,500" in captured.out
        assert "1,000" in captured.out

    def test_main_dispatches_usage(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_usage") as mock_run_usage,
        ):
            config = _make_config()
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "usage"]):
                main()

        mock_run_usage.assert_called_once()


# ---------------------------------------------------------------------------
# _run_audit
# ---------------------------------------------------------------------------


class TestRunAudit:
    def test_main_dispatches_audit(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._run_audit") as mock_run_audit,
            patch("sys.argv", ["aroom", "audit", "purge"]),
        ):
            main()

        mock_run_audit.assert_called_once()

    def test_run_audit_no_action_exits(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_audit

        args = argparse.Namespace(audit_action=None)

        with pytest.raises(SystemExit) as exc_info:
            _run_audit(args)

        assert exc_info.value.code == 1

    def test_run_audit_verify_log_not_found(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_audit

        mock_config = _make_config()

        args = argparse.Namespace(audit_action="verify", audit_file="/tmp/nonexistent-audit.jsonl")

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_audit(args)

        assert exc_info.value.code == 1
        captured = capsys.readouterr()
        assert "not found" in captured.err

    def test_run_audit_verify_no_identity_key(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_audit

        mock_config = _make_config()
        mock_config.identity.private_key = ""

        audit_file = tmp_path / "audit-2024-01-01.jsonl"
        audit_file.write_text('{"event_type":"test"}\n')

        args = argparse.Namespace(audit_action="verify", audit_file=str(audit_file))

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_audit(args)

        assert exc_info.value.code == 1
        captured = capsys.readouterr()
        assert "No identity key" in captured.err

    def test_run_audit_verify_empty_log(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_audit

        mock_config = _make_config()

        audit_file = tmp_path / "audit-2024-01-01.jsonl"
        audit_file.write_text("")

        args = argparse.Namespace(audit_action="verify", audit_file=str(audit_file))

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            patch("anteroom.services.audit.verify_chain", return_value=[]),
        ):
            _run_audit(args)

        captured = capsys.readouterr()
        assert "empty" in captured.out

    def test_run_audit_verify_valid_chain(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_audit

        mock_config = _make_config()

        audit_file = tmp_path / "audit.jsonl"
        audit_file.write_text('{"event_type":"auth"}\n')

        args = argparse.Namespace(audit_action="verify", audit_file=str(audit_file))

        results = [
            {"valid": True, "line": 1, "event_type": "auth", "timestamp": "2024-01-01T00:00:00Z"},
            {"valid": True, "line": 2, "event_type": "tool_call", "timestamp": "2024-01-01T00:01:00Z"},
        ]

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            patch("anteroom.services.audit.verify_chain", return_value=results),
        ):
            _run_audit(args)

        captured = capsys.readouterr()
        assert "INTACT" in captured.out

    def test_run_audit_verify_invalid_entries_exits(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_audit

        mock_config = _make_config()

        audit_file = tmp_path / "audit.jsonl"
        audit_file.write_text('{"event_type":"auth"}\n')

        args = argparse.Namespace(audit_action="verify", audit_file=str(audit_file))

        results = [
            {"valid": True, "line": 1, "event_type": "auth", "timestamp": "2024-01-01T00:00:00Z"},
            {
                "valid": False,
                "line": 2,
                "event_type": "tool_call",
                "timestamp": "2024-01-01T00:01:00Z",
                "error": "hash mismatch",
            },
        ]

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            patch("anteroom.services.audit.verify_chain", return_value=results),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_audit(args)

        assert exc_info.value.code == 1
        captured = capsys.readouterr()
        assert "INVALID" in captured.out

    def test_run_audit_purge_not_enabled_exits(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_audit

        mock_config = _make_config()

        args = argparse.Namespace(audit_action="purge")

        mock_writer = MagicMock()
        mock_writer.enabled = False

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            patch("anteroom.services.audit.create_audit_writer", return_value=mock_writer),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_audit(args)

        assert exc_info.value.code == 1

    def test_run_audit_purge_success(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_audit

        mock_config = _make_config()

        args = argparse.Namespace(audit_action="purge")

        mock_writer = MagicMock()
        mock_writer.enabled = True
        mock_writer.purge_old_logs.return_value = 3

        with (
            patch("anteroom.__main__._load_config_or_exit", return_value=(Path("/tmp"), mock_config, [])),
            patch("anteroom.services.audit.create_audit_writer", return_value=mock_writer),
        ):
            _run_audit(args)

        captured = capsys.readouterr()
        assert "3" in captured.out


# ---------------------------------------------------------------------------
# _validate_pack_ref
# ---------------------------------------------------------------------------


class TestValidatePackRef:
    def test_valid_ref_returns_namespace_name(self) -> None:
        from anteroom.__main__ import _validate_pack_ref

        ns, name = _validate_pack_ref("myns/mypack")
        assert ns == "myns"
        assert name == "mypack"

    def test_missing_slash_defaults_namespace(self) -> None:
        from anteroom.__main__ import _validate_pack_ref

        ns, name = _validate_pack_ref("noSlash")
        assert ns == "default"
        assert name == "noSlash"

    def test_invalid_namespace_exits(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _validate_pack_ref

        with pytest.raises(SystemExit) as exc_info:
            _validate_pack_ref("-invalid/pack")

        assert exc_info.value.code == 1

    def test_invalid_name_exits(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _validate_pack_ref

        with pytest.raises(SystemExit) as exc_info:
            _validate_pack_ref("ns/-invalid")

        assert exc_info.value.code == 1

    def test_name_with_dots_and_dashes_valid(self) -> None:
        from anteroom.__main__ import _validate_pack_ref

        ns, name = _validate_pack_ref("my-ns/my.pack-1")
        assert ns == "my-ns"
        assert name == "my.pack-1"


# ---------------------------------------------------------------------------
# _pick_from_candidates
# ---------------------------------------------------------------------------


class TestPickFromCandidates:
    def test_non_tty_prints_ids(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _pick_from_candidates

        candidates = [{"id": "abc123"}, {"id": "def456"}]

        with patch("sys.stdin") as mock_stdin:
            mock_stdin.isatty.return_value = False
            result = _pick_from_candidates(candidates, "pack", lambda c: c["id"])

        assert result is None
        captured = capsys.readouterr()
        assert "abc123" in captured.err

    def test_tty_valid_selection(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _pick_from_candidates

        candidates = [{"id": "abc123", "name": "First"}, {"id": "def456", "name": "Second"}]

        with (
            patch("sys.stdin") as mock_stdin,
            patch("builtins.input", return_value="1"),
        ):
            mock_stdin.isatty.return_value = True
            result = _pick_from_candidates(candidates, "pack", lambda c: c["name"])

        assert result is candidates[0]

    def test_tty_invalid_selection_returns_none(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _pick_from_candidates

        candidates = [{"id": "abc123"}, {"id": "def456"}]

        with (
            patch("sys.stdin") as mock_stdin,
            patch("builtins.input", return_value="99"),
        ):
            mock_stdin.isatty.return_value = True
            result = _pick_from_candidates(candidates, "pack", lambda c: c["id"])

        assert result is None

    def test_tty_eof_returns_none(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _pick_from_candidates

        candidates = [{"id": "abc123"}]

        with (
            patch("sys.stdin") as mock_stdin,
            patch("builtins.input", side_effect=EOFError),
        ):
            mock_stdin.isatty.return_value = True
            result = _pick_from_candidates(candidates, "pack", lambda c: c["id"])

        assert result is None


# ---------------------------------------------------------------------------
# _run_artifact (dispatch only — full tests in test_artifact_cli.py)
# ---------------------------------------------------------------------------


class TestRunArtifactDispatch:
    def test_artifact_no_action_prints_usage(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_artifact

        mock_config = _make_config()
        args = argparse.Namespace(artifact_action=None)

        with patch("anteroom.db.get_db"):
            _run_artifact(mock_config, args)

        captured = capsys.readouterr()
        assert "Usage" in captured.out

    def test_main_dispatches_artifact(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_artifact") as mock_run,
        ):
            config = _make_config()
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "artifact", "list"]):
                main()

        mock_run.assert_called_once()

    def test_artifact_import_no_flags_exits(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_artifact

        mock_config = _make_config()
        args = argparse.Namespace(
            artifact_action="import",
            skills=False,
            instructions=False,
            import_all=False,
        )

        with (
            patch("anteroom.db.get_db") as mock_get_db,
            pytest.raises(SystemExit) as exc_info,
        ):
            mock_get_db.return_value = MagicMock()
            _run_artifact(mock_config, args)

        assert exc_info.value.code == 1

    def test_artifact_create_success(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_artifact

        mock_config = _make_config()
        mock_config.app.data_dir = tmp_path
        args = argparse.Namespace(
            artifact_action="create",
            type="skill",
            name="my-skill",
            project=False,
        )

        fake_path = tmp_path / "artifacts" / "skill" / "my-skill.yaml"

        with (
            patch("anteroom.db.get_db") as mock_get_db,
            patch("anteroom.services.local_artifacts.scaffold_local_artifact", return_value=fake_path),
        ):
            mock_get_db.return_value = MagicMock()
            _run_artifact(mock_config, args)

        captured = capsys.readouterr()
        assert "Created" in captured.out

    def test_artifact_create_invalid_name_exits(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_artifact

        mock_config = _make_config()
        args = argparse.Namespace(
            artifact_action="create",
            type="skill",
            name="invalid name!",
            project=False,
        )

        with (
            patch("anteroom.db.get_db") as mock_get_db,
            patch(
                "anteroom.services.local_artifacts.scaffold_local_artifact",
                side_effect=ValueError("Invalid name"),
            ),
            pytest.raises(SystemExit) as exc_info,
        ):
            mock_get_db.return_value = MagicMock()
            _run_artifact(mock_config, args)

        assert exc_info.value.code == 1


# ---------------------------------------------------------------------------
# _run_space (dispatch / basic actions)
# ---------------------------------------------------------------------------


class TestRunSpaceDispatch:
    def test_space_no_action_prints_usage(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_space

        mock_config = _make_config()
        args = argparse.Namespace(space_action=None)

        with patch("anteroom.db.get_db"):
            _run_space(mock_config, args)

        captured = capsys.readouterr()
        assert "Usage" in captured.out

    def test_main_dispatches_space(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_space") as mock_run,
        ):
            config = _make_config()
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "space", "list"]):
                main()

        mock_run.assert_called_once()

    def test_space_list_empty(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_space

        mock_config = _make_config()
        args = argparse.Namespace(space_action="list")

        with (
            patch("anteroom.db.get_db") as mock_get_db,
            patch("anteroom.services.space_storage.list_spaces", return_value=[]),
        ):
            mock_get_db.return_value = MagicMock()
            _run_space(mock_config, args)

        captured = capsys.readouterr()
        assert "No spaces found" in captured.out

    def test_space_list_shows_spaces(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_space

        mock_config = _make_config()
        args = argparse.Namespace(space_action="list")

        spaces = [
            {"id": "abc123", "name": "my-space", "file_path": "/home/.anteroom/my-space.yaml", "last_loaded_at": ""},
        ]

        with (
            patch("anteroom.db.get_db") as mock_get_db,
            patch("anteroom.services.space_storage.list_spaces", return_value=spaces),
            patch("anteroom.services.space_storage.count_space_conversations", return_value=0),
            patch("anteroom.services.spaces.is_local_space", return_value=False),
        ):
            mock_get_db.return_value = MagicMock()
            _run_space(mock_config, args)

        captured = capsys.readouterr()
        assert "my-space" in captured.out

    def test_space_delete(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_space

        mock_config = _make_config()
        args = argparse.Namespace(space_action="delete", name="my-space")

        mock_space = {"id": "abc123", "name": "my-space", "file_path": "/tmp/my-space.yaml"}

        with (
            patch("anteroom.db.get_db") as mock_get_db,
            patch("anteroom.services.space_storage.resolve_space", return_value=(mock_space, [])),
            patch("anteroom.services.space_storage.delete_space") as mock_delete,
        ):
            mock_get_db.return_value = MagicMock()
            _run_space(mock_config, args)

        mock_delete.assert_called_once()
        captured = capsys.readouterr()
        assert "Deleted" in captured.out

    def test_space_delete_not_found(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_space

        mock_config = _make_config()
        args = argparse.Namespace(space_action="delete", name="nonexistent")

        with (
            patch("anteroom.db.get_db") as mock_get_db,
            patch("anteroom.services.space_storage.resolve_space", return_value=(None, [])),
        ):
            mock_get_db.return_value = MagicMock()
            _run_space(mock_config, args)

        captured = capsys.readouterr()
        assert "not found" in captured.out

    def test_space_create_invalid_name(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_space

        mock_config = _make_config()
        args = argparse.Namespace(space_action="create", name="invalid name!")

        with patch("anteroom.db.get_db") as mock_get_db:
            mock_get_db.return_value = MagicMock()
            _run_space(mock_config, args)

        captured = capsys.readouterr()
        assert "Invalid space name" in captured.out

    def test_space_create_success(self, capsys: pytest.CaptureFixture[str], tmp_path: Path) -> None:
        from anteroom.__main__ import _run_space

        mock_config = _make_config()
        args = argparse.Namespace(space_action="create", name="my-space")

        mock_space = {"id": "abc12345-0000-0000-0000-000000000000", "name": "my-space"}

        with (
            patch("anteroom.db.get_db") as mock_get_db,
            patch("anteroom.services.spaces.write_space_template"),
            patch("anteroom.services.spaces.compute_file_hash", return_value="fakehash"),
            patch("anteroom.services.space_storage.create_space", return_value=mock_space),
            patch("anteroom.services.space_storage.get_space_by_name", return_value=None),
            patch("anteroom.services.space_storage.sync_space_paths"),
            patch("pathlib.Path.exists", return_value=False),
            patch("pathlib.Path.cwd", return_value=tmp_path),
        ):
            mock_get_db.return_value = MagicMock()
            _run_space(mock_config, args)

        captured = capsys.readouterr()
        assert "Created local space" in captured.out

    def test_space_show_existing(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_space

        mock_config = _make_config()
        args = argparse.Namespace(space_action="show", name="my-space")

        mock_space = {
            "id": "abc12345",
            "name": "my-space",
            "file_path": "/tmp/my-space.yaml",
            "file_hash": "deadbeefdeadbeef",
            "last_loaded_at": "2024-01-01",
            "created_at": "2024-01-01",
        }

        with (
            patch("anteroom.db.get_db") as mock_get_db,
            patch("anteroom.services.space_storage.resolve_space", return_value=(mock_space, [])),
        ):
            mock_get_db.return_value = MagicMock()
            _run_space(mock_config, args)

        captured = capsys.readouterr()
        assert "my-space" in captured.out
        assert "abc12345" in captured.out

    def test_space_load_file_not_found(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_space

        mock_config = _make_config()
        args = argparse.Namespace(space_action="load", path="/nonexistent/space.yaml")

        with patch("anteroom.db.get_db") as mock_get_db:
            mock_get_db.return_value = MagicMock()
            _run_space(mock_config, args)

        captured = capsys.readouterr()
        assert "not found" in captured.out or "Error" in captured.out

    def test_space_map_dir_not_found(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_space

        mock_config = _make_config()
        args = argparse.Namespace(space_action="map", name="my-space", dir_path="/nonexistent/dir")

        mock_space = {"id": "abc123", "name": "my-space", "file_path": "/tmp/my-space.yaml"}

        with (
            patch("anteroom.db.get_db") as mock_get_db,
            patch("anteroom.services.space_storage.resolve_space", return_value=(mock_space, [])),
        ):
            mock_get_db.return_value = MagicMock()
            _run_space(mock_config, args)

        captured = capsys.readouterr()
        assert "not found" in captured.out or "Error" in captured.out


# ---------------------------------------------------------------------------
# main() dispatch — global flags
# ---------------------------------------------------------------------------


class TestMainDispatch:
    def test_main_dispatches_pack(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_pack") as mock_run,
        ):
            config = _make_config()
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "pack", "list"]):
                main()

        mock_run.assert_called_once()

    def test_main_allowed_tools_flag(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_web"),
        ):
            config = _make_config()
            config.safety.allowed_tools = []
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "--allowed-tools", "bash,write_file"]):
                main()

        assert "bash" in config.safety.allowed_tools
        assert "write_file" in config.safety.allowed_tools

    def test_main_allowed_tools_deduplicates(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_web"),
        ):
            config = _make_config()
            config.safety.allowed_tools = ["bash"]
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "--allowed-tools", "bash,write_file"]):
                main()

        assert config.safety.allowed_tools.count("bash") == 1

    def test_main_denied_tools_flag(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_web"),
        ):
            config = _make_config()
            config.safety.denied_tools = []
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "--denied-tools", "bash,run_agent"]):
                main()

        assert "bash" in config.safety.denied_tools
        assert "run_agent" in config.safety.denied_tools

    def test_main_denied_tools_deduplicates(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_web"),
        ):
            config = _make_config()
            config.safety.denied_tools = ["bash"]
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "--denied-tools", "bash,run_agent"]):
                main()

        assert config.safety.denied_tools.count("bash") == 1

    def test_main_read_only_flag(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_web"),
        ):
            config = _make_config()
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "--read-only"]):
                main()

        assert config.safety.read_only is True

    def test_main_read_only_suppressed_when_enforced(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_web"),
        ):
            config = _make_config()
            config.safety.read_only = False
            mock_load.return_value = (Path("/tmp/config.yaml"), config, ["safety.read_only"])
            with patch("sys.argv", ["aroom", "--read-only"]):
                main()

        assert config.safety.read_only is False

    def test_main_temperature_flag(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_web"),
        ):
            config = _make_config()
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "--temperature", "0.7"]):
                main()

        assert config.ai.temperature == pytest.approx(0.7)

    def test_main_temperature_clamped_to_range(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_web"),
        ):
            config = _make_config()
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "--temperature", "5.0"]):
                main()

        assert config.ai.temperature == pytest.approx(2.0)

    def test_main_top_p_flag(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_web"),
        ):
            config = _make_config()
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "--top-p", "0.9"]):
                main()

        assert config.ai.top_p == pytest.approx(0.9)

    def test_main_seed_flag(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_web"),
        ):
            config = _make_config()
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "--seed", "42"]):
                main()

        assert config.ai.seed == 42

    def test_main_approval_mode_auto_warns(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_web"),
        ):
            config = _make_config()
            config.safety.approval_mode = "ask"
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "--approval-mode", "auto"]):
                main()

        assert config.safety.approval_mode == "auto"
        captured = capsys.readouterr()
        assert "Auto-approval" in captured.err

    def test_main_test_flag_calls_test_connection(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__.asyncio.run") as mock_asyncio_run,
        ):
            config = _make_config()
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "--test"]):
                main()

        mock_asyncio_run.assert_called_once()

    def test_main_dispatches_to_web_when_no_command(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_web") as mock_web,
        ):
            config = _make_config()
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom"]):
                main()

        mock_web.assert_called_once()

    def test_main_dispatches_chat(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._run_chat") as mock_chat,
        ):
            config = _make_config()
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            with patch("sys.argv", ["aroom", "chat"]):
                main()

        mock_chat.assert_called_once()

    def test_main_space_flag_resolves_id(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._load_config_or_exit") as mock_load,
            patch("anteroom.__main__._resolve_space_id", return_value="space-456") as mock_resolve,
            patch("anteroom.__main__._run_chat") as mock_chat,
        ):
            config = _make_config()
            mock_load.return_value = (Path("/tmp/config.yaml"), config, [])
            # --space is a global flag; must appear before the subcommand
            with patch("sys.argv", ["aroom", "--space", "my-space", "chat"]):
                main()

        mock_resolve.assert_called_once_with(config, "my-space")
        _, kwargs = mock_chat.call_args
        assert kwargs["space_id"] == "space-456"


# ---------------------------------------------------------------------------
# _resolve_space_id
# ---------------------------------------------------------------------------


class TestResolveIds:
    def test_resolve_space_id_success(self) -> None:
        from anteroom.__main__ import _resolve_space_id

        mock_config = _make_config()
        mock_space = {"id": "space-abc-123"}

        with (
            patch("anteroom.db.get_db") as mock_get_db,
            patch("anteroom.services.space_storage.resolve_space", return_value=(mock_space, [])),
        ):
            mock_get_db.return_value = MagicMock()
            result = _resolve_space_id(mock_config, "my-space")

        assert result == "space-abc-123"

    def test_resolve_space_id_multiple_matches_exits(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _resolve_space_id

        mock_config = _make_config()
        candidates = [{"id": "aaa", "name": "space1"}, {"id": "bbb", "name": "space2"}]

        with (
            patch("anteroom.db.get_db") as mock_get_db,
            patch("anteroom.services.space_storage.resolve_space", return_value=(None, candidates)),
            pytest.raises(SystemExit) as exc_info,
        ):
            mock_get_db.return_value = MagicMock()
            _resolve_space_id(mock_config, "space")

        assert exc_info.value.code == 1

    def test_resolve_space_id_not_found_exits(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _resolve_space_id

        mock_config = _make_config()

        with (
            patch("anteroom.db.get_db") as mock_get_db,
            patch("anteroom.services.space_storage.resolve_space", return_value=(None, [])),
            pytest.raises(SystemExit) as exc_info,
        ):
            mock_get_db.return_value = MagicMock()
            _resolve_space_id(mock_config, "nonexistent-space")

        assert exc_info.value.code == 1


# ---------------------------------------------------------------------------
# _run_chat
# ---------------------------------------------------------------------------


class TestRunChat:
    def test_run_chat_invalid_project_path_exits(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_chat

        mock_config = _make_config()

        with pytest.raises(SystemExit) as exc_info:
            _run_chat(mock_config, project_path="/nonexistent/directory/that/does/not/exist")

        assert exc_info.value.code == 1
        captured = capsys.readouterr()
        assert "not a directory" in captured.err

    def test_run_chat_model_override(self) -> None:
        from anteroom.__main__ import _run_chat

        mock_config = _make_config()

        with (
            patch("anteroom.cli.repl.run_cli"),
            patch("anteroom.__main__.asyncio.run"),
        ):
            _run_chat(mock_config, model="gpt-4-turbo")

        assert mock_config.ai.model == "gpt-4-turbo"

    def test_run_chat_keyboard_interrupt_suppressed(self) -> None:
        from anteroom.__main__ import _run_chat

        mock_config = _make_config()

        with patch("anteroom.__main__.asyncio.run", side_effect=KeyboardInterrupt):
            _run_chat(mock_config)

    def test_run_chat_api_connection_error_exits(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_chat

        mock_config = _make_config()

        class FakeAPIConnectionError(Exception):
            pass

        FakeAPIConnectionError.__name__ = "APIConnectionError"

        with (
            patch("anteroom.__main__.asyncio.run", side_effect=FakeAPIConnectionError("conn refused")),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_chat(mock_config)

        assert exc_info.value.code == 1


# ---------------------------------------------------------------------------
# _run_exec
# ---------------------------------------------------------------------------


class TestRunExec:
    def test_run_exec_keyboard_interrupt_exits_130(self) -> None:
        from anteroom.__main__ import _run_exec

        mock_config = _make_config()

        with (
            patch("anteroom.__main__.asyncio.run", side_effect=KeyboardInterrupt),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_exec(mock_config, prompt="test")

        assert exc_info.value.code == 130

    def test_run_exec_timeout_clamped(self) -> None:
        from anteroom.__main__ import _run_exec

        mock_config = _make_config()

        with patch("anteroom.__main__.asyncio.run", return_value=0) as mock_run:
            with pytest.raises(SystemExit):
                _run_exec(mock_config, prompt="test", timeout=5.0)

        mock_run.assert_called_once()

    def test_run_exec_model_override(self) -> None:
        from anteroom.__main__ import _run_exec

        mock_config = _make_config()

        with patch("anteroom.__main__.asyncio.run", return_value=0):
            with pytest.raises(SystemExit):
                _run_exec(mock_config, prompt="test", model="gpt-4")

        assert mock_config.ai.model == "gpt-4"

    def test_run_exec_api_connection_error_exits_1(self, capsys: pytest.CaptureFixture[str]) -> None:
        from anteroom.__main__ import _run_exec

        mock_config = _make_config()

        class FakeAPIConnectionError(Exception):
            pass

        FakeAPIConnectionError.__name__ = "APIConnectionError"

        with (
            patch("anteroom.__main__.asyncio.run", side_effect=FakeAPIConnectionError("conn refused")),
            pytest.raises(SystemExit) as exc_info,
        ):
            _run_exec(mock_config, prompt="test")

        assert exc_info.value.code == 1


# ---------------------------------------------------------------------------
# _run_pack (dispatch only — full tests in test_pack_cli.py)
# ---------------------------------------------------------------------------


class TestRunPackDispatch:
    def test_main_dispatches_pack_install(self) -> None:
        from anteroom.__main__ import main

        with (
            patch("anteroom.__main__._run_pack_dispatch") as mock_dispatch,
        ):
            with patch("sys.argv", ["aroom", "pack", "install", "/tmp/pack"]):
                main()

        mock_dispatch.assert_called_once()
        args_passed = mock_dispatch.call_args[0][0]
        assert args_passed.pack_action == "install"


# ---------------------------------------------------------------------------
# aroom unpack
# ---------------------------------------------------------------------------


class TestUnpackDispatch:
    def test_main_dispatches_unpack(self) -> None:
        from anteroom.__main__ import main

        with patch("anteroom.unpack.unpack") as mock_unpack:
            with patch("sys.argv", ["aroom", "unpack", "/tmp/dest"]):
                main()

        mock_unpack.assert_called_once_with("/tmp/dest", force=False)

    def test_main_dispatches_unpack_with_force(self) -> None:
        from anteroom.__main__ import main

        with patch("anteroom.unpack.unpack") as mock_unpack:
            with patch("sys.argv", ["aroom", "unpack", "/tmp/dest", "--force"]):
                main()

        mock_unpack.assert_called_once_with("/tmp/dest", force=True)



"""Tests for the DOCX tool."""

from __future__ import annotations

from unittest.mock import patch

import pytest

from anteroom.tools.office_docx import _MAX_CONTENT_BLOCKS, AVAILABLE, DEFINITION, handle

_needs_docx = pytest.mark.skipif(not AVAILABLE, reason="requires python-docx: pip install anteroom[office]")


@pytest.fixture(autouse=True)
def _set_working_dir(tmp_path, monkeypatch):
    monkeypatch.chdir(tmp_path)
    yield


class TestDefinition:
    def test_name(self):
        assert DEFINITION["name"] == "docx"

    def test_required_params(self):
        assert "action" in DEFINITION["parameters"]["required"]
        assert "path" in DEFINITION["parameters"]["required"]


@_needs_docx
class TestCreate:
    @pytest.mark.asyncio
    async def test_create_simple(self, tmp_path):
        result = await handle(
            action="create",
            path="test.docx",
            content_blocks=[
                {"type": "heading", "text": "Title", "level": 1},
                {"type": "paragraph", "text": "Hello world"},
            ],
        )
        assert "error" not in result
        assert result["blocks_written"] == 2
        assert (tmp_path / "test.docx").exists()

    @pytest.mark.asyncio
    async def test_create_with_table(self, tmp_path):
        result = await handle(
            action="create",
            path="tables.docx",
            content_blocks=[
                {"type": "table", "rows": [["A", "B"], ["1", "2"]]},
            ],
        )
        assert "error" not in result
        assert (tmp_path / "tables.docx").exists()

    @pytest.mark.asyncio
    async def test_create_no_blocks(self):
        result = await handle(action="create", path="empty.docx")
        assert "error" in result

    @pytest.mark.asyncio
    async def test_create_too_many_blocks(self):
        blocks = [{"type": "paragraph", "text": f"p{i}"} for i in range(_MAX_CONTENT_BLOCKS + 1)]
        result = await handle(action="create", path="big.docx", content_blocks=blocks)
        assert "error" in result
        assert "Too many" in result["error"]

    @pytest.mark.asyncio
    async def test_create_subdirectory(self, tmp_path):
        result = await handle(
            action="create",
            path="sub/dir/test.docx",
            content_blocks=[{"type": "paragraph", "text": "nested"}],
        )
        assert "error" not in result
        assert (tmp_path / "sub" / "dir" / "test.docx").exists()


@_needs_docx
class TestRead:
    @pytest.mark.asyncio
    async def test_read_simple(self, tmp_path):
        await handle(
            action="create",
            path="read_me.docx",
            content_blocks=[
                {"type": "heading", "text": "Title", "level": 1},
                {"type": "paragraph", "text": "Body text"},
            ],
        )
        result = await handle(action="read", path="read_me.docx")
        assert "error" not in result
        assert "Title" in result["content"]
        assert "Body text" in result["content"]

    @pytest.mark.asyncio
    async def test_read_with_table(self, tmp_path):
        await handle(
            action="create",
            path="table.docx",
            content_blocks=[
                {"type": "table", "rows": [["Name", "Age"], ["Alice", "30"]]},
            ],
        )
        result = await handle(action="read", path="table.docx")
        assert "error" not in result
        assert "Alice" in result["content"]

    @pytest.mark.asyncio
    async def test_read_not_found(self):
        result = await handle(action="read", path="missing.docx")
        assert "error" in result
        assert "not found" in result["error"].lower()

    @pytest.mark.asyncio
    async def test_read_corrupt_file(self, tmp_path):
        corrupt = tmp_path / "corrupt.docx"
        corrupt.write_bytes(b"not a docx file")
        result = await handle(action="read", path="corrupt.docx")
        assert "error" in result


@_needs_docx
class TestEdit:
    @pytest.mark.asyncio
    async def test_edit_replace(self, tmp_path):
        await handle(
            action="create",
            path="edit_me.docx",
            content_blocks=[{"type": "paragraph", "text": "Hello world"}],
        )
        result = await handle(
            action="edit",
            path="edit_me.docx",
            replacements=[{"old": "Hello", "new": "Goodbye"}],
        )
        assert "error" not in result
        assert result["replacements_made"] >= 1

        read_result = await handle(action="read", path="edit_me.docx")
        assert "Goodbye" in read_result["content"]

    @pytest.mark.asyncio
    async def test_edit_append(self, tmp_path):
        await handle(
            action="create",
            path="append.docx",
            content_blocks=[{"type": "paragraph", "text": "Original"}],
        )
        result = await handle(
            action="edit",
            path="append.docx",
            content_blocks=[{"type": "paragraph", "text": "Appended"}],
        )
        assert "error" not in result
        assert result["blocks_appended"] == 1

    @pytest.mark.asyncio
    async def test_edit_not_found(self):
        result = await handle(
            action="edit",
            path="missing.docx",
            replacements=[{"old": "a", "new": "b"}],
        )
        assert "error" in result

    @pytest.mark.asyncio
    async def test_edit_no_operations(self, tmp_path):
        await handle(
            action="create",
            path="noop.docx",
            content_blocks=[{"type": "paragraph", "text": "text"}],
        )
        result = await handle(action="edit", path="noop.docx")
        assert "error" in result

    @pytest.mark.asyncio
    async def test_edit_too_many_blocks(self, tmp_path):
        await handle(
            action="create",
            path="big.docx",
            content_blocks=[{"type": "paragraph", "text": "text"}],
        )
        blocks = [{"type": "paragraph", "text": f"p{i}"} for i in range(_MAX_CONTENT_BLOCKS + 1)]
        result = await handle(action="edit", path="big.docx", content_blocks=blocks)
        assert "error" in result


@_needs_docx
class TestPathValidation:
    @pytest.mark.asyncio
    async def test_blocked_system_path_rejected(self):
        result = await handle(
            action="create",
            path="/etc/shadow",
            content_blocks=[{"type": "paragraph", "text": "x"}],
        )
        assert "error" in result

    @pytest.mark.asyncio
    async def test_null_bytes_rejected(self):
        result = await handle(action="read", path="test\x00.docx")
        assert "error" in result


@_needs_docx
class TestUnknownAction:
    @pytest.mark.asyncio
    async def test_unknown_action(self):
        result = await handle(action="delete", path="test.docx")
        assert "error" in result
        assert "Unknown action" in result["error"]


class TestGracefulDegradation:
    @pytest.mark.asyncio
    async def test_unavailable(self):
        with patch("anteroom.tools.office_docx.AVAILABLE", False):
            result = await handle(action="read", path="test.docx")
            assert "error" in result
            assert "pip install" in result["error"]


# ---------------------------------------------------------------------------
# New action tests (lib backend)
# ---------------------------------------------------------------------------


@_needs_docx
class TestHeadersFooters:
    async def _create_doc(self, name: str = "hf.docx") -> None:
        await handle(
            action="create",
            path=name,
            content_blocks=[{"type": "paragraph", "text": "Body text"}],
        )

    @pytest.mark.asyncio
    async def test_set_header_and_footer(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="headers_footers",
            path="hf.docx",
            operation="set",
            header_text="My Header",
            footer_text="My Footer",
        )
        assert "error" not in result
        assert "Set headers/footers" in result["result"]

    @pytest.mark.asyncio
    async def test_read_header_and_footer(self, tmp_path):
        await self._create_doc()
        await handle(
            action="headers_footers",
            path="hf.docx",
            operation="set",
            header_text="Test Header",
            footer_text="Test Footer",
        )
        result = await handle(
            action="headers_footers",
            path="hf.docx",
            operation="read",
        )
        assert "error" not in result
        assert "Test Header" in result["header"]
        assert "Test Footer" in result["footer"]
        assert result["section"] == 1

    @pytest.mark.asyncio
    async def test_read_default_empty(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="headers_footers",
            path="hf.docx",
            operation="read",
        )
        assert "error" not in result
        assert result["section"] == 1

    @pytest.mark.asyncio
    async def test_set_header_only(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="headers_footers",
            path="hf.docx",
            operation="set",
            header_text="Only Header",
        )
        assert "error" not in result

    @pytest.mark.asyncio
    async def test_set_footer_only(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="headers_footers",
            path="hf.docx",
            operation="set",
            footer_text="Only Footer",
        )
        assert "error" not in result

    @pytest.mark.asyncio
    async def test_invalid_section_index(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="headers_footers",
            path="hf.docx",
            operation="read",
            section_index=99,
        )
        assert "error" in result
        assert "Section 99 not found" in result["error"]

    @pytest.mark.asyncio
    async def test_unknown_operation(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="headers_footers",
            path="hf.docx",
            operation="delete",
        )
        assert "error" in result
        assert "Unknown operation" in result["error"]

    @pytest.mark.asyncio
    async def test_file_not_found(self):
        result = await handle(
            action="headers_footers",
            path="missing.docx",
            operation="read",
        )
        assert "error" in result
        assert "not found" in result["error"].lower()


@_needs_docx
class TestInsertImage:
    async def _create_doc(self, name: str = "img.docx") -> None:
        await handle(
            action="create",
            path=name,
            content_blocks=[{"type": "paragraph", "text": "Before image"}],
        )

    def _create_test_png(self, tmp_path, name: str = "test.png") -> str:
        """Create a minimal valid PNG file (1x1 red pixel)."""
        import struct
        import zlib

        def _chunk(chunk_type: bytes, data: bytes) -> bytes:
            c = chunk_type + data
            crc = struct.pack(">I", zlib.crc32(c) & 0xFFFFFFFF)
            return struct.pack(">I", len(data)) + c + crc

        sig = b"\x89PNG\r\n\x1a\n"
        ihdr_data = struct.pack(">IIBBBBB", 1, 1, 8, 2, 0, 0, 0)
        raw_row = b"\x00\xff\x00\x00"  # filter byte + RGB
        idat_data = zlib.compress(raw_row)

        png = sig + _chunk(b"IHDR", ihdr_data) + _chunk(b"IDAT", idat_data) + _chunk(b"IEND", b"")
        path = tmp_path / name
        path.write_bytes(png)
        return name

    @pytest.mark.asyncio
    async def test_insert_image_basic(self, tmp_path):
        await self._create_doc()
        img_name = self._create_test_png(tmp_path)
        result = await handle(
            action="insert_image",
            path="img.docx",
            image_path=img_name,
        )
        assert "error" not in result
        assert "Inserted image" in result["result"]
        assert result["path"] == "img.docx"

    @pytest.mark.asyncio
    async def test_insert_image_with_width(self, tmp_path):
        await self._create_doc()
        img_name = self._create_test_png(tmp_path)
        result = await handle(
            action="insert_image",
            path="img.docx",
            image_path=img_name,
            width_inches=3.0,
        )
        assert "error" not in result

    @pytest.mark.asyncio
    async def test_insert_image_missing_path(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="insert_image",
            path="img.docx",
        )
        assert "error" in result
        assert "image_path is required" in result["error"]

    @pytest.mark.asyncio
    async def test_insert_image_not_found(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="insert_image",
            path="img.docx",
            image_path="nonexistent.png",
        )
        assert "error" in result
        assert "not found" in result["error"].lower()

    @pytest.mark.asyncio
    async def test_insert_image_doc_not_found(self):
        result = await handle(
            action="insert_image",
            path="missing.docx",
            image_path="test.png",
        )
        assert "error" in result
        assert "not found" in result["error"].lower()


@_needs_docx
class TestStyles:
    async def _create_doc(self, name: str = "styles.docx") -> None:
        await handle(
            action="create",
            path=name,
            content_blocks=[
                {"type": "heading", "text": "Title", "level": 1},
                {"type": "paragraph", "text": "Normal paragraph"},
            ],
        )

    @pytest.mark.asyncio
    async def test_list_styles(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="styles",
            path="styles.docx",
            operation="list",
        )
        assert "error" not in result
        assert "styles" in result
        assert result["count"] > 0
        style_names = [s["name"] for s in result["styles"]]
        assert any("Normal" in n for n in style_names)

    @pytest.mark.asyncio
    async def test_list_styles_default_operation(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="styles",
            path="styles.docx",
        )
        assert "error" not in result
        assert "styles" in result

    @pytest.mark.asyncio
    async def test_read_style(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="styles",
            path="styles.docx",
            operation="read",
            style_name="Normal",
        )
        assert "error" not in result
        assert result["style"]["name"] == "Normal"

    @pytest.mark.asyncio
    async def test_read_style_not_found(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="styles",
            path="styles.docx",
            operation="read",
            style_name="NonexistentStyleXYZ",
        )
        assert "error" in result
        assert "not found" in result["error"]

    @pytest.mark.asyncio
    async def test_read_style_missing_name(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="styles",
            path="styles.docx",
            operation="read",
        )
        assert "error" in result
        assert "style_name required" in result["error"]

    @pytest.mark.asyncio
    async def test_set_style(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="styles",
            path="styles.docx",
            operation="set",
            style_name="Heading 2",
            paragraph_index=1,
        )
        assert "error" not in result
        assert "Applied style" in result["result"]

    @pytest.mark.asyncio
    async def test_set_style_out_of_range(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="styles",
            path="styles.docx",
            operation="set",
            style_name="Normal",
            paragraph_index=999,
        )
        assert "error" in result
        assert "out of range" in result["error"]

    @pytest.mark.asyncio
    async def test_set_style_missing_params(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="styles",
            path="styles.docx",
            operation="set",
        )
        assert "error" in result
        assert "style_name and paragraph_index required" in result["error"]

    @pytest.mark.asyncio
    async def test_unknown_operation(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="styles",
            path="styles.docx",
            operation="delete",
        )
        assert "error" in result
        assert "Unknown operation" in result["error"]

    @pytest.mark.asyncio
    async def test_file_not_found(self):
        result = await handle(
            action="styles",
            path="missing.docx",
            operation="list",
        )
        assert "error" in result


@_needs_docx
class TestPageSetup:
    async def _create_doc(self, name: str = "page.docx") -> None:
        await handle(
            action="create",
            path=name,
            content_blocks=[{"type": "paragraph", "text": "Content"}],
        )

    @pytest.mark.asyncio
    async def test_read_page_setup(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="page_setup",
            path="page.docx",
            operation="read",
        )
        assert "error" not in result
        assert result["result"] == "Read page setup"
        assert "orientation" in result
        assert "top_margin" in result
        assert "bottom_margin" in result
        assert "left_margin" in result
        assert "right_margin" in result
        assert "page_width" in result
        assert "page_height" in result

    @pytest.mark.asyncio
    async def test_set_orientation_landscape(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="page_setup",
            path="page.docx",
            orientation="landscape",
        )
        assert "error" not in result
        assert "Updated page setup" in result["result"]

        read_result = await handle(
            action="page_setup",
            path="page.docx",
            operation="read",
        )
        assert read_result["orientation"] == "landscape"

    @pytest.mark.asyncio
    async def test_set_orientation_portrait(self, tmp_path):
        await self._create_doc()
        # First set to landscape, then back to portrait
        await handle(
            action="page_setup",
            path="page.docx",
            orientation="landscape",
        )
        result = await handle(
            action="page_setup",
            path="page.docx",
            orientation="portrait",
        )
        assert "error" not in result
        read_result = await handle(
            action="page_setup",
            path="page.docx",
            operation="read",
        )
        assert read_result["orientation"] == "portrait"

    @pytest.mark.asyncio
    async def test_set_margins(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="page_setup",
            path="page.docx",
            margins={"top": 0.5, "bottom": 0.5, "left": 0.75, "right": 0.75},
        )
        assert "error" not in result
        assert "Updated page setup" in result["result"]

    @pytest.mark.asyncio
    async def test_set_paper_size(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="page_setup",
            path="page.docx",
            paper_size="a4",
        )
        assert "error" not in result

    @pytest.mark.asyncio
    async def test_file_not_found(self):
        result = await handle(
            action="page_setup",
            path="missing.docx",
            operation="read",
        )
        assert "error" in result


@_needs_docx
class TestSections:
    async def _create_doc(self, name: str = "sections.docx") -> None:
        await handle(
            action="create",
            path=name,
            content_blocks=[{"type": "paragraph", "text": "Section 1 content"}],
        )

    @pytest.mark.asyncio
    async def test_list_sections(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="sections",
            path="sections.docx",
            operation="list",
        )
        assert "error" not in result
        assert result["count"] >= 1
        assert "sections" in result
        section = result["sections"][0]
        assert "index" in section
        assert "start_type" in section
        assert "page_width" in section
        assert "page_height" in section

    @pytest.mark.asyncio
    async def test_list_sections_default_operation(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="sections",
            path="sections.docx",
        )
        assert "error" not in result
        assert "sections" in result

    @pytest.mark.asyncio
    async def test_add_section(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="sections",
            path="sections.docx",
            operation="add",
            start_type="new_page",
        )
        assert "error" not in result
        assert "Added new_page section break" in result["result"]
        assert result["total_sections"] == 2

    @pytest.mark.asyncio
    async def test_add_continuous_section(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="sections",
            path="sections.docx",
            operation="add",
            start_type="continuous",
        )
        assert "error" not in result
        assert "Added continuous section break" in result["result"]

    @pytest.mark.asyncio
    async def test_add_section_default_type(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="sections",
            path="sections.docx",
            operation="add",
        )
        assert "error" not in result
        assert "new_page" in result["result"]

    @pytest.mark.asyncio
    async def test_unknown_operation(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="sections",
            path="sections.docx",
            operation="delete",
        )
        assert "error" in result
        assert "Unknown operation" in result["error"]

    @pytest.mark.asyncio
    async def test_file_not_found(self):
        result = await handle(
            action="sections",
            path="missing.docx",
            operation="list",
        )
        assert "error" in result


@_needs_docx
class TestFindRegex:
    async def _create_doc(self, name: str = "regex.docx") -> None:
        await handle(
            action="create",
            path=name,
            content_blocks=[
                {"type": "paragraph", "text": "Hello world"},
                {"type": "paragraph", "text": "Hello again"},
                {"type": "paragraph", "text": "Goodbye world"},
            ],
        )

    @pytest.mark.asyncio
    async def test_find_simple_pattern(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="find_regex",
            path="regex.docx",
            pattern="Hello",
        )
        assert "error" not in result
        assert result["count"] == 2
        assert len(result["matches"]) == 2
        assert result["truncated"] is False

    @pytest.mark.asyncio
    async def test_find_regex_pattern(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="find_regex",
            path="regex.docx",
            pattern=r"Hello\s\w+",
        )
        assert "error" not in result
        assert result["count"] == 2
        for match in result["matches"]:
            assert "text" in match
            assert "paragraph" in match
            assert "start" in match
            assert "end" in match

    @pytest.mark.asyncio
    async def test_find_no_matches(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="find_regex",
            path="regex.docx",
            pattern="ZZZZZ",
        )
        assert "error" not in result
        assert result["count"] == 0
        assert result["matches"] == []

    @pytest.mark.asyncio
    async def test_find_and_replace(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="find_regex",
            path="regex.docx",
            pattern="Hello",
            replace_with="Hi",
        )
        assert "error" not in result
        assert result["replacements"] >= 2
        assert result["path"] == "regex.docx"

        read_result = await handle(action="read", path="regex.docx")
        assert "Hi" in read_result["content"]
        assert "Hello" not in read_result["content"]

    @pytest.mark.asyncio
    async def test_find_replace_regex(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="find_regex",
            path="regex.docx",
            pattern=r"Hello\s(\w+)",
            replace_with=r"Greetings \1",
        )
        assert "error" not in result
        assert result["replacements"] >= 1

    @pytest.mark.asyncio
    async def test_missing_pattern(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="find_regex",
            path="regex.docx",
        )
        assert "error" in result
        assert "pattern is required" in result["error"]

    @pytest.mark.asyncio
    async def test_invalid_regex(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="find_regex",
            path="regex.docx",
            pattern="[invalid",
        )
        assert "error" in result
        assert "Invalid regex" in result["error"]

    @pytest.mark.asyncio
    async def test_pattern_too_long(self, tmp_path):
        await self._create_doc()
        result = await handle(
            action="find_regex",
            path="regex.docx",
            pattern="a" * 201,
        )
        assert "error" in result
        assert "too long" in result["error"]

    @pytest.mark.asyncio
    async def test_file_not_found(self):
        result = await handle(
            action="find_regex",
            path="missing.docx",
            pattern="test",
        )
        assert "error" in result


# ---------------------------------------------------------------------------
# COM-only action tests (verify they return the COM-only error on lib backend)
# ---------------------------------------------------------------------------


@_needs_docx
class TestTrackChangesCOMOnly:
    @pytest.mark.asyncio
    async def test_returns_com_only_error(self, tmp_path):
        await handle(
            action="create",
            path="tc.docx",
            content_blocks=[{"type": "paragraph", "text": "text"}],
        )
        result = await handle(action="track_changes", path="tc.docx")
        assert "error" in result
        assert "COM backend" in result["error"]


@_needs_docx
class TestCommentsCOMOnly:
    @pytest.mark.asyncio
    async def test_returns_com_only_error(self, tmp_path):
        await handle(
            action="create",
            path="comments.docx",
            content_blocks=[{"type": "paragraph", "text": "text"}],
        )
        result = await handle(action="comments", path="comments.docx")
        assert "error" in result
        assert "COM backend" in result["error"]


@_needs_docx
class TestBookmarksCOMOnly:
    @pytest.mark.asyncio
    async def test_returns_com_only_error(self, tmp_path):
        await handle(
            action="create",
            path="bm.docx",
            content_blocks=[{"type": "paragraph", "text": "text"}],
        )
        result = await handle(action="bookmarks", path="bm.docx")
        assert "error" in result
        assert "COM backend" in result["error"]


@_needs_docx
class TestTocCOMOnly:
    @pytest.mark.asyncio
    async def test_returns_com_only_error(self, tmp_path):
        await handle(
            action="create",
            path="toc.docx",
            content_blocks=[{"type": "paragraph", "text": "text"}],
        )
        result = await handle(action="toc", path="toc.docx")
        assert "error" in result
        assert "COM backend" in result["error"]


@_needs_docx
class TestExportPdfCOMOnly:
    @pytest.mark.asyncio
    async def test_returns_com_only_error(self, tmp_path):
        await handle(
            action="create",
            path="export.docx",
            content_blocks=[{"type": "paragraph", "text": "text"}],
        )
        result = await handle(action="export_pdf", path="export.docx")
        assert "error" in result
        assert "COM backend" in result["error"]


class TestComDispatchErrorHandling:
    @pytest.mark.asyncio
    async def test_dispatch_com_returns_error_dict_on_exception(self):
        from unittest.mock import AsyncMock, MagicMock

        mock_manager = MagicMock()
        mock_manager.run_com = AsyncMock(side_effect=RuntimeError("Access denied by security policy"))
        mock_com_mod = MagicMock()
        mock_com_mod.get_manager.return_value = mock_manager
        mock_com_mod.COM_AVAILABLE = True

        with (
            patch("anteroom.tools.office_docx.AVAILABLE", True),
            patch("anteroom.tools.office_docx._BACKEND", "com"),
            patch("anteroom.tools.office_docx._com_mod", mock_com_mod),
        ):
            result = await handle(action="edit", path="test.docx", replacements=[{"old": "a", "new": "b"}])

        assert "error" in result
        assert "Access denied by security policy" in result["error"]
        assert "RuntimeError" in result["error"]



"""Tests for CLI approval prompt logic."""

from __future__ import annotations

from io import StringIO
from unittest.mock import AsyncMock, MagicMock, patch

import pytest

# ---------------------------------------------------------------------------
# _apply_choice logic tests
# ---------------------------------------------------------------------------
# _apply_choice is a nested function inside _confirm_destructive, so we
# cannot import it directly. Instead we replicate the logic and test it.
# ---------------------------------------------------------------------------


def _apply_choice_logic(choice: str) -> str:
    """Replicate the approval choice mapping for unit testing."""
    if choice in ("a", "always"):
        return "always"
    if choice in ("s", "session"):
        return "session"
    if choice in ("y", "yes"):
        return "once"
    return "denied"


class TestApplyChoiceLogic:
    def test_allow_once_y(self) -> None:
        assert _apply_choice_logic("y") == "once"

    def test_allow_once_yes(self) -> None:
        assert _apply_choice_logic("yes") == "once"

    def test_session_s(self) -> None:
        assert _apply_choice_logic("s") == "session"

    def test_session_word(self) -> None:
        assert _apply_choice_logic("session") == "session"

    def test_always_a(self) -> None:
        assert _apply_choice_logic("a") == "always"

    def test_always_word(self) -> None:
        assert _apply_choice_logic("always") == "always"

    def test_deny_n(self) -> None:
        assert _apply_choice_logic("n") == "denied"

    def test_deny_no(self) -> None:
        assert _apply_choice_logic("no") == "denied"

    def test_deny_empty(self) -> None:
        assert _apply_choice_logic("") == "denied"

    def test_deny_invalid(self) -> None:
        assert _apply_choice_logic("xyz") == "denied"

    def test_case_insensitive_via_lower(self) -> None:
        assert _apply_choice_logic("Y".lower()) == "once"
        assert _apply_choice_logic("YES".lower()) == "once"
        assert _apply_choice_logic("S".lower()) == "session"
        assert _apply_choice_logic("A".lower()) == "always"
        assert _apply_choice_logic("N".lower()) == "denied"


# ---------------------------------------------------------------------------
# write_raw tests
# ---------------------------------------------------------------------------


class TestWriteRaw:
    def test_write_raw_writes_to_stdout(self) -> None:
        from anteroom.cli import renderer

        fake_fd = StringIO()
        original = renderer._stdout
        try:
            renderer._stdout = fake_fd
            renderer.write_raw("hello prompt")
            assert fake_fd.getvalue() == "hello prompt"
        finally:
            renderer._stdout = original

    def test_write_raw_noop_without_stdout(self) -> None:
        from anteroom.cli import renderer

        original = renderer._stdout
        try:
            renderer._stdout = None
            renderer.write_raw("should not crash")
        finally:
            renderer._stdout = original


# ---------------------------------------------------------------------------
# _sub_prompt_async building block tests
# ---------------------------------------------------------------------------


class TestSubPromptAsync:
    @pytest.mark.asyncio
    async def test_nested_session_prompt_returns_stripped(self) -> None:
        """Verify nested PromptSession returns stripped input."""
        mock_session_cls = MagicMock()
        mock_instance = MagicMock()
        mock_instance.prompt_async = AsyncMock(return_value="  y  ")
        mock_session_cls.return_value = mock_instance

        with patch("prompt_toolkit.PromptSession", mock_session_cls):
            from prompt_toolkit import PromptSession

            sub = PromptSession()
            raw = await sub.prompt_async("  > ")
            answer = raw.strip() if raw is not None else None

        assert answer == "y"

    @pytest.mark.asyncio
    async def test_nested_session_empty_returns_empty_string(self) -> None:
        """Verify pressing Enter with no text returns empty string, not None."""
        mock_session_cls = MagicMock()
        mock_instance = MagicMock()
        mock_instance.prompt_async = AsyncMock(return_value="")
        mock_session_cls.return_value = mock_instance

        with patch("prompt_toolkit.PromptSession", mock_session_cls):
            from prompt_toolkit import PromptSession

            sub = PromptSession()
            raw = await sub.prompt_async("  > ")
            answer = raw.strip() if raw is not None else None

        assert answer == ""
        assert answer is not None

    @pytest.mark.asyncio
    async def test_nested_session_eof_returns_none(self) -> None:
        """Verify EOF (Ctrl-D) on nested session is handled."""
        mock_session_cls = MagicMock()
        mock_instance = MagicMock()
        mock_instance.prompt_async = AsyncMock(side_effect=EOFError)
        mock_session_cls.return_value = mock_instance

        with patch("prompt_toolkit.PromptSession", mock_session_cls):
            from prompt_toolkit import PromptSession

            sub = PromptSession()
            try:
                await sub.prompt_async("  > ")
                answer: str | None = ""
            except EOFError:
                answer = None

        assert answer is None

    @pytest.mark.asyncio
    async def test_nested_session_keyboard_interrupt_returns_none(self) -> None:
        """Verify Ctrl-C on nested session is handled."""
        mock_session_cls = MagicMock()
        mock_instance = MagicMock()
        mock_instance.prompt_async = AsyncMock(side_effect=KeyboardInterrupt)
        mock_session_cls.return_value = mock_instance

        with patch("prompt_toolkit.PromptSession", mock_session_cls):
            from prompt_toolkit import PromptSession

            sub = PromptSession()
            try:
                await sub.prompt_async("  > ")
                answer: str | None = ""
            except KeyboardInterrupt:
                answer = None

        assert answer is None



"""Tests for tree-sitter codebase index service.

Covers all branches including tree-sitter mocked paths. See issue #689.
"""

from __future__ import annotations

import tempfile
from pathlib import Path
from typing import Any
from unittest.mock import MagicMock, patch

import pytest

from anteroom.services.codebase_index import (
    CodebaseIndexService,
    CodebaseIndexUnavailableError,
    CodebaseMap,
    FileSymbols,
    SymbolInfo,
    _classify_node,
    _estimate_tokens,
    _extract_imports_python,
    _extract_name,
    _extract_signature,
    _parse_file,
    _rank_files,
    create_index_service,
)


class TestClassifyNode:
    def test_import_types(self) -> None:
        assert _classify_node("import_statement", "python") == "import"
        assert _classify_node("import_from_statement", "python") == "import"
        assert _classify_node("use_declaration", "rust") == "import"

    def test_class_types(self) -> None:
        assert _classify_node("class_definition", "python") == "class"
        assert _classify_node("class_declaration", "java") == "class"
        assert _classify_node("struct_item", "rust") == "class"
        assert _classify_node("module", "ruby") == "class"

    def test_interface_type(self) -> None:
        assert _classify_node("interface_declaration", "typescript") == "interface"

    def test_type_alias(self) -> None:
        assert _classify_node("type_alias_declaration", "typescript") == "type"

    def test_type_declaration(self) -> None:
        assert _classify_node("type_declaration", "go") == "type"

    def test_enum_type(self) -> None:
        assert _classify_node("enum_item", "rust") == "enum"

    def test_method_type(self) -> None:
        assert _classify_node("method_declaration", "java") == "method"

    def test_function_type(self) -> None:
        assert _classify_node("function_definition", "python") == "function"
        assert _classify_node("function_declaration", "javascript") == "function"
        assert _classify_node("impl_item", "rust") == "function"

    def test_export_type(self) -> None:
        assert _classify_node("export_statement", "javascript") == "export"

    def test_unknown_type(self) -> None:
        assert _classify_node("some_random_node", "python") == "symbol"


class TestExtractImportsPython:
    def test_from_import(self) -> None:
        node = MagicMock()
        node.text = b"from os.path import join"
        result = _extract_imports_python(node)
        assert result == ["os.path"]

    def test_plain_import(self) -> None:
        node = MagicMock()
        node.text = b"import os, sys"
        result = _extract_imports_python(node)
        assert result == ["os", "sys"]

    def test_import_with_alias(self) -> None:
        node = MagicMock()
        node.text = b"import numpy as np"
        result = _extract_imports_python(node)
        assert result == ["numpy"]

    def test_empty_from(self) -> None:
        node = MagicMock()
        node.text = b"from "
        result = _extract_imports_python(node)
        # "from " splits to ["from", ""] — len < 2 after strip, so empty
        assert result == []


class TestEstimateTokens:
    def test_fallback_char_estimate(self) -> None:
        # tiktoken is installed, so mock it away to test the fallback
        with patch.dict("sys.modules", {"tiktoken": None}):
            # Force the except branch by making import fail
            original_import = __builtins__.__import__ if hasattr(__builtins__, "__import__") else __import__

            def _mock_import(name: str, *args: Any, **kwargs: Any) -> Any:
                if name == "tiktoken":
                    raise ImportError("mocked")
                return original_import(name, *args, **kwargs)

            with patch("builtins.__import__", side_effect=_mock_import):
                text = "a" * 400
                result = _estimate_tokens(text)
                assert result == 100

    def test_tiktoken_if_available(self) -> None:
        # tiktoken is a dependency, so it should work
        result = _estimate_tokens("hello world")
        assert isinstance(result, int)
        assert result > 0


class TestRankFiles:
    def test_files_ranked_by_import_count(self) -> None:
        files = [
            FileSymbols(path="src/a.py", language="python", symbols=[], imports=["b"]),
            FileSymbols(path="src/b.py", language="python", symbols=[], imports=[]),
            FileSymbols(path="src/c.py", language="python", symbols=[], imports=["b"]),
        ]
        ranked = _rank_files(files)
        # b.py is imported by a.py and c.py, so should be first
        assert ranked[0].path == "src/b.py"

    def test_ties_sorted_alphabetically(self) -> None:
        files = [
            FileSymbols(path="src/z.py", language="python", symbols=[], imports=[]),
            FileSymbols(path="src/a.py", language="python", symbols=[], imports=[]),
        ]
        ranked = _rank_files(files)
        assert ranked[0].path == "src/a.py"
        assert ranked[1].path == "src/z.py"

    def test_empty_list(self) -> None:
        assert _rank_files([]) == []


class TestCodebaseIndexService:
    def test_is_available_without_treesitter(self) -> None:
        with patch.dict("sys.modules", {"tree_sitter": None}):
            service = CodebaseIndexService()
            service._available = None
            # Force reimport failure
            with patch("builtins.__import__", side_effect=ImportError("no tree_sitter")):
                assert service.is_available() is False

    def test_is_available_caches_result(self) -> None:
        service = CodebaseIndexService()
        service._available = True
        assert service.is_available() is True

    def test_scan_fallback_without_treesitter(self) -> None:
        service = CodebaseIndexService()
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            (Path(tmpdir) / ".git").mkdir()  # project marker
            (Path(tmpdir) / "hello.py").write_text("def greet(): pass\n")
            result = service.scan(tmpdir)

        assert isinstance(result, CodebaseMap)
        assert len(result.files) == 1
        assert result.files[0].path == "hello.py"
        assert result.files[0].language == "python"
        assert result.files[0].symbols == []  # fallback = no symbols

    def test_scan_respects_exclude_dirs(self) -> None:
        service = CodebaseIndexService(exclude_dirs=["hidden"])
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            (Path(tmpdir) / ".git").mkdir()  # project marker
            (Path(tmpdir) / "hello.py").write_text("x = 1\n")
            hidden = Path(tmpdir) / "hidden"
            hidden.mkdir()
            (hidden / "secret.py").write_text("y = 2\n")
            result = service.scan(tmpdir)

        paths = [f.path for f in result.files]
        assert "hello.py" in paths
        assert "hidden/secret.py" not in paths

    def test_scan_respects_language_filter(self) -> None:
        service = CodebaseIndexService(languages=["python"])
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            (Path(tmpdir) / ".git").mkdir()  # project marker
            (Path(tmpdir) / "hello.py").write_text("x = 1\n")
            (Path(tmpdir) / "app.js").write_text("let x = 1;\n")
            result = service.scan(tmpdir)

        paths = [f.path for f in result.files]
        assert "hello.py" in paths
        assert "app.js" not in paths

    def test_scan_skips_non_project_directory(self) -> None:
        service = CodebaseIndexService()
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            # No project markers — should skip scan entirely
            (Path(tmpdir) / "hello.py").write_text("x = 1\n")
            result = service.scan(tmpdir)

        assert isinstance(result, CodebaseMap)
        assert result.files == []

    def test_scan_runs_with_pyproject_toml_marker(self) -> None:
        service = CodebaseIndexService()
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            (Path(tmpdir) / "pyproject.toml").write_text("[project]\nname = 'test'\n")
            (Path(tmpdir) / "hello.py").write_text("x = 1\n")
            result = service.scan(tmpdir)

        assert len(result.files) == 1
        assert result.files[0].path == "hello.py"

    def test_scan_runs_with_package_json_marker(self) -> None:
        service = CodebaseIndexService()
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            (Path(tmpdir) / "package.json").write_text("{}\n")
            (Path(tmpdir) / "app.js").write_text("let x = 1;\n")
            result = service.scan(tmpdir)

        assert len(result.files) == 1
        assert result.files[0].path == "app.js"

    def test_scan_runs_with_cargo_toml_marker(self) -> None:
        service = CodebaseIndexService()
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            (Path(tmpdir) / "Cargo.toml").write_text("[package]\nname = 'test'\n")
            (Path(tmpdir) / "main.rs").write_text("fn main() {}\n")
            result = service.scan(tmpdir)

        assert len(result.files) == 1
        assert result.files[0].path == "main.rs"
        assert result.files[0].language == "rust"

    def test_scan_runs_with_go_mod_marker(self) -> None:
        service = CodebaseIndexService()
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            (Path(tmpdir) / "go.mod").write_text("module example.com/test\n")
            (Path(tmpdir) / "main.go").write_text("package main\n")
            result = service.scan(tmpdir)

        assert len(result.files) == 1
        assert result.files[0].path == "main.go"
        assert result.files[0].language == "go"

    def test_scan_runs_with_makefile_marker(self) -> None:
        service = CodebaseIndexService()
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            (Path(tmpdir) / "Makefile").write_text("all:\n\techo hello\n")
            (Path(tmpdir) / "lib.py").write_text("x = 1\n")
            result = service.scan(tmpdir)

        assert len(result.files) == 1
        assert result.files[0].path == "lib.py"

    def test_scan_runs_with_cmakelists_marker(self) -> None:
        service = CodebaseIndexService()
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            (Path(tmpdir) / "CMakeLists.txt").write_text("cmake_minimum_required(VERSION 3.10)\n")
            (Path(tmpdir) / "main.c").write_text("int main() { return 0; }\n")
            result = service.scan(tmpdir)

        assert len(result.files) == 1
        assert result.files[0].path == "main.c"
        assert result.files[0].language == "c"

    def test_scan_skip_populates_scan_time(self) -> None:
        service = CodebaseIndexService()
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            (Path(tmpdir) / "hello.py").write_text("x = 1\n")
            result = service.scan(tmpdir)

        assert result.scan_time >= 0.0

    def test_scan_skip_sets_correct_root(self) -> None:
        service = CodebaseIndexService()
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            result = service.scan(tmpdir)

        assert result.root == str(Path(tmpdir).resolve())

    def test_get_map_returns_empty_for_non_project(self) -> None:
        service = CodebaseIndexService()
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            (Path(tmpdir) / "hello.py").write_text("x = 1\n")
            result = service.get_map(tmpdir, token_budget=5000)

        assert result == ""

    def test_scan_with_deeply_nested_files_and_no_marker(self) -> None:
        service = CodebaseIndexService()
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            deep = Path(tmpdir) / "a" / "b" / "c"
            deep.mkdir(parents=True)
            (deep / "deep.py").write_text("x = 1\n")
            result = service.scan(tmpdir)

        assert result.files == []

    def test_format_map_basic(self) -> None:
        service = CodebaseIndexService()
        cmap = CodebaseMap(
            root="/tmp/test",
            files=[
                FileSymbols(
                    path="src/main.py",
                    language="python",
                    symbols=[
                        SymbolInfo(name="greet", kind="function", signature="def greet(name: str) -> str:"),
                        SymbolInfo(name="os", kind="import", signature="import os"),
                    ],
                ),
            ],
        )
        result = service.format_map(cmap, token_budget=5000)
        assert "src/main.py" in result
        assert "def greet(name: str) -> str:" in result
        assert "import os" not in result  # imports are excluded from output

    def test_format_map_respects_token_budget(self) -> None:
        service = CodebaseIndexService()
        files = []
        for i in range(100):
            files.append(
                FileSymbols(
                    path=f"src/module_{i}.py",
                    language="python",
                    symbols=[
                        SymbolInfo(
                            name=f"func_{i}",
                            kind="function",
                            signature=f"def func_{i}(x: int, y: int, z: int) -> dict[str, Any]:",
                        ),
                    ],
                )
            )
        cmap = CodebaseMap(root="/tmp/test", files=files)
        result = service.format_map(cmap, token_budget=200)
        # Should not include all 100 files
        assert result.count("## src/module_") < 100

    def test_format_map_empty_symbols(self) -> None:
        service = CodebaseIndexService()
        cmap = CodebaseMap(
            root="/tmp/test",
            files=[FileSymbols(path="src/empty.py", language="python", symbols=[])],
        )
        result = service.format_map(cmap, token_budget=5000)
        assert "src/empty.py" in result
        assert "(no exported symbols)" in result

    def test_get_map_wraps_in_xml(self) -> None:
        service = CodebaseIndexService()
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            (Path(tmpdir) / ".git").mkdir()  # project marker
            (Path(tmpdir) / "hello.py").write_text("x = 1\n")
            result = service.get_map(tmpdir, token_budget=5000)

        assert result.startswith("\n<codebase_index>")
        assert result.endswith("</codebase_index>")

    def test_get_map_empty_dir(self) -> None:
        service = CodebaseIndexService()
        service._available = False

        with tempfile.TemporaryDirectory() as tmpdir:
            result = service.get_map(tmpdir)

        assert result == ""


class TestCreateIndexService:
    def test_returns_service_when_enabled(self) -> None:
        config = MagicMock()
        config.codebase_index.enabled = True
        config.codebase_index.exclude_dirs = [".git", "node_modules"]
        config.codebase_index.languages = []
        result = create_index_service(config)
        assert isinstance(result, CodebaseIndexService)

    def test_returns_none_when_disabled(self) -> None:
        config = MagicMock()
        config.codebase_index.enabled = False
        result = create_index_service(config)
        assert result is None

    def test_passes_languages_filter(self) -> None:
        config = MagicMock()
        config.codebase_index.enabled = True
        config.codebase_index.exclude_dirs = []
        config.codebase_index.languages = ["python", "typescript"]
        result = create_index_service(config)
        assert result is not None
        assert result._languages == {"python", "typescript"}

    def test_none_languages_means_all(self) -> None:
        config = MagicMock()
        config.codebase_index.enabled = True
        config.codebase_index.exclude_dirs = []
        config.codebase_index.languages = None
        result = create_index_service(config)
        assert result is not None
        assert result._languages is None


class TestExtractName:
    """Tests for _extract_name — covers lines 125-132."""

    def _make_child(self, child_type: str, text: str | bytes) -> MagicMock:
        child = MagicMock()
        child.type = child_type
        child.text = text
        return child

    def test_identifier_child_bytes(self) -> None:
        node = MagicMock()
        node.type = "function_definition"
        node.children = [self._make_child("identifier", b"my_func")]
        assert _extract_name(node, "python") == "my_func"

    def test_identifier_child_str(self) -> None:
        node = MagicMock()
        node.type = "function_definition"
        node.children = [self._make_child("identifier", "my_func")]
        assert _extract_name(node, "python") == "my_func"

    def test_name_child_type(self) -> None:
        node = MagicMock()
        node.type = "method"
        node.children = [self._make_child("name", b"do_thing")]
        assert _extract_name(node, "ruby") == "do_thing"

    def test_type_identifier_child(self) -> None:
        node = MagicMock()
        node.type = "struct_item"
        node.children = [self._make_child("type_identifier", b"MyStruct")]
        assert _extract_name(node, "rust") == "MyStruct"

    def test_property_identifier_child(self) -> None:
        node = MagicMock()
        node.type = "class_declaration"
        node.children = [self._make_child("property_identifier", b"MyClass")]
        assert _extract_name(node, "typescript") == "MyClass"

    def test_import_node_uses_full_text_bytes(self) -> None:
        node = MagicMock()
        node.type = "import_statement"
        node.text = b"import os\nimport sys"
        node.children = []
        result = _extract_name(node, "python")
        assert result == "import os"

    def test_import_node_uses_full_text_str(self) -> None:
        node = MagicMock()
        node.type = "import_from_statement"
        node.text = "from pathlib import Path\nmore"
        node.children = []
        result = _extract_name(node, "python")
        assert result == "from pathlib import Path"

    def test_use_declaration_node(self) -> None:
        node = MagicMock()
        node.type = "use_declaration"
        node.text = b"use std::collections::HashMap;"
        node.children = []
        result = _extract_name(node, "rust")
        assert result == "use std::collections::HashMap;"

    def test_no_matching_child_returns_empty(self) -> None:
        node = MagicMock()
        node.type = "block"
        node.text = b"{ body }"
        node.children = [self._make_child("punctuation", b"{")]
        assert _extract_name(node, "python") == ""

    def test_import_text_truncated_at_80(self) -> None:
        node = MagicMock()
        node.type = "import_statement"
        long_text = "import " + "a" * 100
        node.text = long_text.encode()
        node.children = []
        result = _extract_name(node, "python")
        assert len(result) == 80


class TestExtractSignature:
    """Tests for _extract_signature — covers lines 137-142."""

    def test_single_line_bytes(self) -> None:
        node = MagicMock()
        node.text = b"def greet(name: str) -> str:"
        result = _extract_signature(node, "python", "function")
        assert result == "def greet(name: str) -> str:"

    def test_single_line_str(self) -> None:
        node = MagicMock()
        node.text = "def greet(name: str) -> str:"
        result = _extract_signature(node, "python", "function")
        assert result == "def greet(name: str) -> str:"

    def test_multiline_extracts_first_line(self) -> None:
        node = MagicMock()
        node.text = b"def foo():\n    return 1\n"
        result = _extract_signature(node, "python", "function")
        assert result == "def foo():"

    def test_long_signature_truncated(self) -> None:
        node = MagicMock()
        long_sig = "def " + "a" * 120 + "(x):"
        node.text = long_sig.encode()
        result = _extract_signature(node, "python", "function")
        assert len(result) == 120
        assert result.endswith("...")

    def test_exactly_120_chars_not_truncated(self) -> None:
        node = MagicMock()
        sig = "x" * 120
        node.text = sig.encode()
        result = _extract_signature(node, "python", "function")
        assert result == sig
        assert not result.endswith("...")


class TestParseFile:
    """Tests for _parse_file — covers lines 175-214."""

    def _make_mock_parser(self, root_node: Any) -> MagicMock:
        parser = MagicMock()
        tree = MagicMock()
        tree.root_node = root_node
        parser.parse.return_value = tree
        return parser

    def _make_leaf_node(self, node_type: str, text: bytes, children: list | None = None) -> MagicMock:
        node = MagicMock()
        node.type = node_type
        node.text = text
        node.children = children or []
        return node

    def test_returns_none_for_oversized_file(self, tmp_path: Path) -> None:
        f = tmp_path / "big.py"
        f.write_bytes(b"x")
        parser = MagicMock()
        ts_lang = MagicMock()
        with patch("anteroom.services.codebase_index._MAX_FILE_SIZE", 0):
            result = _parse_file(f, tmp_path, "python", parser, ts_lang)
        assert result is None

    def test_returns_none_on_oserror(self, tmp_path: Path) -> None:
        f = tmp_path / "missing.py"
        parser = MagicMock()
        ts_lang = MagicMock()
        result = _parse_file(f, tmp_path, "python", parser, ts_lang)
        assert result is None

    def test_returns_file_symbols_for_unknown_language(self, tmp_path: Path) -> None:
        f = tmp_path / "app.unknown"
        f.write_bytes(b"some content")
        root_node = self._make_leaf_node("root", b"", children=[])
        parser = self._make_mock_parser(root_node)
        ts_lang = MagicMock()
        # "unknown_lang" has no query types in _SYMBOL_QUERIES
        result = _parse_file(f, tmp_path, "unknown_lang", parser, ts_lang)
        assert result is not None
        assert result.symbols == []
        assert result.imports == []
        assert result.language == "unknown_lang"

    def test_extracts_python_function(self, tmp_path: Path) -> None:
        f = tmp_path / "mod.py"
        f.write_bytes(b"def hello(): pass\n")

        id_child = self._make_leaf_node("identifier", b"hello")
        func_node = self._make_leaf_node("function_definition", b"def hello(): pass", children=[id_child])
        root_node = self._make_leaf_node("module", b"def hello(): pass\n", children=[func_node])

        parser = self._make_mock_parser(root_node)
        ts_lang = MagicMock()
        result = _parse_file(f, tmp_path, "python", parser, ts_lang)

        assert result is not None
        assert len(result.symbols) == 1
        assert result.symbols[0].name == "hello"
        assert result.symbols[0].kind == "function"

    def test_extracts_python_import_and_adds_to_imports(self, tmp_path: Path) -> None:
        f = tmp_path / "mod.py"
        f.write_bytes(b"import os\n")

        import_node = self._make_leaf_node("import_statement", b"import os", children=[])
        root_node = self._make_leaf_node("module", b"import os\n", children=[import_node])

        parser = self._make_mock_parser(root_node)
        ts_lang = MagicMock()
        result = _parse_file(f, tmp_path, "python", parser, ts_lang)

        assert result is not None
        # import kind with no name is still added as a symbol
        assert any(s.kind == "import" for s in result.symbols)
        assert "os" in result.imports

    def test_extracts_non_python_import_with_name(self, tmp_path: Path) -> None:
        f = tmp_path / "app.go"
        f.write_bytes(b'import "fmt"\n')

        id_child = self._make_leaf_node("identifier", b"fmt")
        import_node = self._make_leaf_node("import_declaration", b'import "fmt"', children=[id_child])
        root_node = self._make_leaf_node("source_file", b'import "fmt"\n', children=[import_node])

        parser = self._make_mock_parser(root_node)
        ts_lang = MagicMock()
        result = _parse_file(f, tmp_path, "go", parser, ts_lang)

        assert result is not None
        assert "fmt" in result.imports

    def test_import_without_name_not_added_to_imports(self, tmp_path: Path) -> None:
        f = tmp_path / "app.go"
        f.write_bytes(b"import .\n")

        # import_declaration node with no identifier child and no extractable name
        import_node = self._make_leaf_node("import_declaration", b"import .", children=[])
        root_node = self._make_leaf_node("source_file", b"import .\n", children=[import_node])

        parser = self._make_mock_parser(root_node)
        ts_lang = MagicMock()
        result = _parse_file(f, tmp_path, "go", parser, ts_lang)

        assert result is not None
        # The import_declaration text doesn't start with "import " (no space after dot), so
        # _extract_name returns the full text; but the node type contains "import" so kind=="import"
        # imports list may be empty since the text doesn't parse cleanly as a module name
        assert isinstance(result.imports, list)

    def test_symbol_without_name_and_non_import_not_added(self, tmp_path: Path) -> None:
        f = tmp_path / "mod.py"
        f.write_bytes(b"def (): pass\n")

        # function_definition node but no identifier child and not an import
        func_node = self._make_leaf_node("function_definition", b"def (): pass", children=[])
        root_node = self._make_leaf_node("module", b"def (): pass\n", children=[func_node])

        parser = self._make_mock_parser(root_node)
        ts_lang = MagicMock()
        result = _parse_file(f, tmp_path, "python", parser, ts_lang)

        assert result is not None
        # name is empty and kind is "function" (not import), so symbol should not be added
        assert len(result.symbols) == 0

    def test_sets_mtime(self, tmp_path: Path) -> None:
        f = tmp_path / "mod.py"
        f.write_bytes(b"x = 1\n")
        root_node = self._make_leaf_node("module", b"x = 1\n", children=[])
        parser = self._make_mock_parser(root_node)
        ts_lang = MagicMock()
        result = _parse_file(f, tmp_path, "python", parser, ts_lang)
        assert result is not None
        assert result.mtime == f.stat().st_mtime

    def test_sets_relative_path(self, tmp_path: Path) -> None:
        subdir = tmp_path / "src"
        subdir.mkdir()
        f = subdir / "mod.py"
        f.write_bytes(b"x = 1\n")
        root_node = self._make_leaf_node("module", b"x = 1\n", children=[])
        parser = self._make_mock_parser(root_node)
        ts_lang = MagicMock()
        result = _parse_file(f, tmp_path, "python", parser, ts_lang)
        assert result is not None
        assert result.path == "src/mod.py"

    def test_recursive_walk_into_children(self, tmp_path: Path) -> None:
        f = tmp_path / "mod.py"
        f.write_bytes(b"class Foo:\n    def bar(self): pass\n")

        id_child = self._make_leaf_node("identifier", b"bar")
        method_node = self._make_leaf_node("function_definition", b"def bar(self): pass", children=[id_child])
        class_id = self._make_leaf_node("identifier", b"Foo")
        class_body = b"class Foo:\n    def bar(self): pass"
        class_node = self._make_leaf_node("class_definition", class_body, children=[class_id, method_node])
        root_node = self._make_leaf_node("module", b"class Foo:\n    def bar(self): pass\n", children=[class_node])

        parser = self._make_mock_parser(root_node)
        ts_lang = MagicMock()
        result = _parse_file(f, tmp_path, "python", parser, ts_lang)

        assert result is not None
        names = [s.name for s in result.symbols]
        assert "Foo" in names
        assert "bar" in names


class TestRankFilesInitPath:
    """Tests for _rank_files __init__ path — covers line 234."""

    def test_init_py_path_stripped_and_ranked(self) -> None:
        # src/pkg/__init__.py -> module "src.pkg" (after stripping .__init__)
        # When another file imports "src.pkg", __init__.py gets a count
        files = [
            FileSymbols(path="src/pkg/__init__.py", language="python", symbols=[], imports=[]),
            FileSymbols(path="src/main.py", language="python", symbols=[], imports=["src.pkg"]),
        ]
        ranked = _rank_files(files)
        # src/pkg/__init__.py is imported by src/main.py -> count=1, ranked first
        assert ranked[0].path == "src/pkg/__init__.py"

    def test_init_py_module_name_derived_without_init_suffix(self) -> None:
        # Verify the .__init__ stripping works: "a.b.__init__" -> "a.b"
        # An importer using "a.b" should hit the __init__.py
        files = [
            FileSymbols(path="a/b/__init__.py", language="python", symbols=[], imports=[]),
            FileSymbols(path="a/b/util.py", language="python", symbols=[], imports=[]),
            FileSymbols(path="consumer.py", language="python", symbols=[], imports=["a.b"]),
        ]
        ranked = _rank_files(files)
        # a/b/__init__.py maps to module "a.b" which matches the import "a.b"
        assert ranked[0].path == "a/b/__init__.py"

    def test_non_init_file_uses_stem(self) -> None:
        # A regular file like "utils.py" -> stem "utils"
        # When another file imports "utils", it should get count 1 and rank first
        files = [
            FileSymbols(path="utils.py", language="python", symbols=[], imports=[]),
            FileSymbols(path="app.py", language="python", symbols=[], imports=["utils"]),
        ]
        ranked = _rank_files(files)
        assert ranked[0].path == "utils.py"


class TestEnsureParserAndGetLanguage:
    """Tests for _ensure_parser and _get_language — covers lines 278, 282-283, 292-302, 310."""

    def test_ensure_parser_returns_cached(self) -> None:
        service = CodebaseIndexService()
        mock_parser = MagicMock()
        service._parser = mock_parser
        result = service._ensure_parser()
        assert result is mock_parser

    def test_ensure_parser_creates_parser_when_tree_sitter_available(self) -> None:
        service = CodebaseIndexService()
        service._parser = None

        mock_ts = MagicMock()
        mock_parser_instance = MagicMock()
        mock_ts.Parser.return_value = mock_parser_instance

        with patch.dict("sys.modules", {"tree_sitter": mock_ts}):
            result = service._ensure_parser()

        assert result is mock_parser_instance
        assert service._parser is mock_parser_instance

    def test_ensure_parser_raises_when_tree_sitter_missing(self) -> None:
        service = CodebaseIndexService()
        service._parser = None

        with patch.dict("sys.modules", {"tree_sitter": None}):
            with pytest.raises(CodebaseIndexUnavailableError, match="tree-sitter is not installed"):
                service._ensure_parser()

        assert service._available is False

    def test_get_language_returns_cached(self) -> None:
        service = CodebaseIndexService()
        mock_lang = MagicMock()
        service._lang_cache["python"] = mock_lang
        result = service._get_language("python")
        assert result is mock_lang

    def test_get_language_loads_from_pack(self) -> None:
        service = CodebaseIndexService()
        mock_pack = MagicMock()
        mock_lang = MagicMock()
        mock_pack.get_language.return_value = mock_lang

        with patch.dict("sys.modules", {"tree_sitter_language_pack": mock_pack}):
            result = service._get_language("python")

        assert result is mock_lang
        assert service._lang_cache["python"] is mock_lang

    def test_get_language_returns_none_on_import_error(self) -> None:
        service = CodebaseIndexService()

        with patch.dict("sys.modules", {"tree_sitter_language_pack": None}):
            result = service._get_language("python")

        assert result is None

    def test_get_language_returns_none_on_other_exception(self) -> None:
        service = CodebaseIndexService()
        mock_pack = MagicMock()
        mock_pack.get_language.side_effect = Exception("unknown lang")

        with patch.dict("sys.modules", {"tree_sitter_language_pack": mock_pack}):
            result = service._get_language("python")

        assert result is None

    def test_is_available_sets_true_when_parser_succeeds(self) -> None:
        service = CodebaseIndexService()
        service._available = None
        mock_ts = MagicMock()
        mock_ts.Parser.return_value = MagicMock()

        with patch.dict("sys.modules", {"tree_sitter": mock_ts}):
            result = service.is_available()

        assert result is True
        assert service._available is True


class TestScanWithTreeSitter:
    """Tests for scan() with tree-sitter mocked — covers lines 348-375."""

    def _make_mock_tree_sitter(self) -> tuple[MagicMock, MagicMock]:
        """Return (mock_ts_module, mock_parser)."""
        mock_ts = MagicMock()
        mock_parser = MagicMock()
        mock_ts.Parser.return_value = mock_parser
        return mock_ts, mock_parser

    def _setup_parse_result(self, mock_parser: MagicMock, symbols: list[SymbolInfo] | None = None) -> None:
        """Configure parser to return an empty tree."""
        root_node = MagicMock()
        root_node.type = "module"
        root_node.children = []
        tree = MagicMock()
        tree.root_node = root_node
        mock_parser.parse.return_value = tree

    def test_scan_uses_mtime_cache(self, tmp_path: Path) -> None:
        (tmp_path / ".git").mkdir()
        f = tmp_path / "mod.py"
        f.write_bytes(b"x = 1\n")

        service = CodebaseIndexService()
        mock_ts, mock_parser = self._make_mock_tree_sitter()
        self._setup_parse_result(mock_parser)

        mock_lang = MagicMock()

        with patch.dict("sys.modules", {"tree_sitter": mock_ts, "tree_sitter_language_pack": MagicMock()}):
            service._parser = mock_parser
            service._available = True
            service._lang_cache["python"] = mock_lang

            # First scan — parses and caches
            result1 = service.scan(str(tmp_path))
            assert len(result1.files) == 1
            parse_count_after_first = mock_parser.parse.call_count

            # Second scan with same mtime — should use cache, not re-parse
            result2 = service.scan(str(tmp_path))
            assert len(result2.files) == 1
            assert mock_parser.parse.call_count == parse_count_after_first  # no new parse calls

    def test_scan_skips_file_on_oserror_during_mtime(self, tmp_path: Path) -> None:
        (tmp_path / ".git").mkdir()
        f = tmp_path / "mod.py"
        f.write_bytes(b"x = 1\n")

        service = CodebaseIndexService()
        mock_ts, mock_parser = self._make_mock_tree_sitter()
        mock_lang = MagicMock()

        # Patch stat only for the source file path by replacing it in the scan loop
        real_stat = Path.stat

        def stat_raises_for_source(self: Path, **kwargs: Any) -> Any:  # type: ignore[override]
            if self == f:
                raise OSError("no access")
            return real_stat(self, **kwargs)

        with patch.dict("sys.modules", {"tree_sitter": mock_ts, "tree_sitter_language_pack": MagicMock()}):
            service._parser = mock_parser
            service._available = True
            service._lang_cache["python"] = mock_lang

            with patch.object(Path, "stat", stat_raises_for_source):
                result = service.scan(str(tmp_path))

        # OSError on stat for source file means the file is skipped
        assert result.files == []

    def test_scan_adds_filename_only_when_language_unavailable(self, tmp_path: Path) -> None:
        (tmp_path / ".git").mkdir()
        f = tmp_path / "mod.py"
        f.write_bytes(b"x = 1\n")

        service = CodebaseIndexService()
        mock_ts, mock_parser = self._make_mock_tree_sitter()

        with patch.dict("sys.modules", {"tree_sitter": mock_ts, "tree_sitter_language_pack": MagicMock()}):
            service._parser = mock_parser
            service._available = True
            # No lang in cache and _get_language returns None
            with patch.object(service, "_get_language", return_value=None):
                result = service.scan(str(tmp_path))

        assert len(result.files) == 1
        assert result.files[0].path == "mod.py"
        assert result.files[0].symbols == []

    def test_scan_skips_parse_file_returning_none(self, tmp_path: Path) -> None:
        (tmp_path / ".git").mkdir()
        f = tmp_path / "mod.py"
        f.write_bytes(b"x = 1\n")

        service = CodebaseIndexService()
        mock_ts, mock_parser = self._make_mock_tree_sitter()
        mock_lang = MagicMock()

        with patch.dict("sys.modules", {"tree_sitter": mock_ts, "tree_sitter_language_pack": MagicMock()}):
            service._parser = mock_parser
            service._available = True
            service._lang_cache["python"] = mock_lang
            # Make _parse_file return None (e.g. oversized file)
            with patch("anteroom.services.codebase_index._parse_file", return_value=None):
                result = service.scan(str(tmp_path))

        assert result.files == []

    def test_scan_full_pipeline_with_mocked_treesitter(self, tmp_path: Path) -> None:
        (tmp_path / ".git").mkdir()
        f = tmp_path / "mod.py"
        f.write_bytes(b"def hello(): pass\n")

        id_child = MagicMock()
        id_child.type = "identifier"
        id_child.text = b"hello"
        id_child.children = []

        func_node = MagicMock()
        func_node.type = "function_definition"
        func_node.text = b"def hello(): pass"
        func_node.children = [id_child]

        root_node = MagicMock()
        root_node.type = "module"
        root_node.children = [func_node]

        mock_ts, mock_parser = self._make_mock_tree_sitter()
        tree = MagicMock()
        tree.root_node = root_node
        mock_parser.parse.return_value = tree
        mock_lang = MagicMock()

        with patch.dict("sys.modules", {"tree_sitter": mock_ts, "tree_sitter_language_pack": MagicMock()}):
            service = CodebaseIndexService()
            service._parser = mock_parser
            service._available = True
            service._lang_cache["python"] = mock_lang
            result = service.scan(str(tmp_path))

        assert len(result.files) == 1
        assert result.files[0].path == "mod.py"
        assert len(result.files[0].symbols) == 1
        assert result.files[0].symbols[0].name == "hello"



"""Tests for the search router."""

from __future__ import annotations

from unittest.mock import AsyncMock, MagicMock, patch

from fastapi import FastAPI
from fastapi.testclient import TestClient

from anteroom.routers.search import router


def _make_app(*, vec_enabled: bool = False, embedding_service=None) -> FastAPI:
    """Create a minimal FastAPI app with the search router."""
    app = FastAPI()
    app.include_router(router, prefix="/api")

    # Minimal state
    app.state.vec_enabled = vec_enabled
    app.state.embedding_service = embedding_service

    # Mock db
    mock_db = MagicMock()
    mock_db_manager = MagicMock()
    mock_db_manager.get.return_value = mock_db
    app.state.db = mock_db
    app.state.db_manager = mock_db_manager

    return app


class TestUnifiedSearch:
    def test_keyword_mode_uses_fts(self) -> None:
        app = _make_app()
        with patch("anteroom.routers.search.storage") as mock_storage:
            mock_storage.list_conversations.return_value = [{"id": "c1", "title": "Test Conv", "message_count": 5}]
            client = TestClient(app)
            resp = client.get("/api/search?q=hello&mode=keyword")
            assert resp.status_code == 200
            data = resp.json()
            assert data["mode"] == "keyword"
            assert len(data["results"]) == 1

    def test_auto_mode_falls_back_to_keyword(self) -> None:
        app = _make_app(vec_enabled=False)
        with patch("anteroom.routers.search.storage") as mock_storage:
            mock_storage.list_conversations.return_value = []
            client = TestClient(app)
            resp = client.get("/api/search?q=test&mode=auto")
            assert resp.status_code == 200
            assert resp.json()["mode"] == "keyword"

    def test_semantic_mode_errors_when_unavailable(self) -> None:
        app = _make_app(vec_enabled=False)
        client = TestClient(app)
        resp = client.get("/api/search?q=test&mode=semantic")
        assert resp.status_code == 503

    def test_auto_mode_uses_semantic_when_available(self) -> None:
        service = AsyncMock()
        service.embed = AsyncMock(return_value=[0.1] * 1536)
        app = _make_app(vec_enabled=True, embedding_service=service)

        with patch("anteroom.routers.search.storage") as mock_storage:
            mock_storage.search_similar_messages.return_value = [
                {
                    "message_id": "m1",
                    "conversation_id": "c1",
                    "content": "Hello",
                    "role": "user",
                    "distance": 0.1,
                }
            ]
            client = TestClient(app)
            resp = client.get("/api/search?q=hello&mode=auto")
            assert resp.status_code == 200
            data = resp.json()
            assert data["mode"] == "semantic"

    def test_missing_query_returns_422(self) -> None:
        app = _make_app()
        client = TestClient(app)
        resp = client.get("/api/search")
        assert resp.status_code == 422

    def test_invalid_mode_returns_422(self) -> None:
        app = _make_app()
        client = TestClient(app)
        resp = client.get("/api/search?q=test&mode=invalid")
        assert resp.status_code == 422


class TestSemanticSearch:
    def test_returns_grouped_results(self) -> None:
        service = AsyncMock()
        service.embed = AsyncMock(return_value=[0.1] * 1536)
        app = _make_app(vec_enabled=True, embedding_service=service)

        with patch("anteroom.routers.search.storage") as mock_storage:
            mock_storage.search_similar_messages.return_value = [
                {"message_id": "m1", "conversation_id": "c1", "content": "Hello", "role": "user", "distance": 0.1},
                {"message_id": "m2", "conversation_id": "c1", "content": "World", "role": "assistant", "distance": 0.2},
            ]
            mock_storage.get_conversation.return_value = {"title": "Test Conv"}

            client = TestClient(app)
            resp = client.get("/api/search/semantic?q=hello")
            assert resp.status_code == 200
            data = resp.json()
            assert len(data["results"]) == 1  # grouped by conversation
            assert data["results"][0]["conversation_id"] == "c1"
            assert len(data["results"][0]["messages"]) == 2

    def test_errors_when_service_unavailable(self) -> None:
        app = _make_app(vec_enabled=True, embedding_service=None)
        client = TestClient(app)
        resp = client.get("/api/search/semantic?q=hello")
        assert resp.status_code == 503

    def test_errors_when_vec_not_loaded(self) -> None:
        service = AsyncMock()
        app = _make_app(vec_enabled=False, embedding_service=service)
        client = TestClient(app)
        resp = client.get("/api/search/semantic?q=hello")
        assert resp.status_code == 503

    def test_errors_when_embedding_fails(self) -> None:
        service = AsyncMock()
        service.embed = AsyncMock(return_value=None)
        app = _make_app(vec_enabled=True, embedding_service=service)

        client = TestClient(app)
        resp = client.get("/api/search/semantic?q=hello")
        assert resp.status_code == 500


class TestSearchTypeFilter:
    """Type filter parameter on unified search endpoint."""

    def test_keyword_search_with_type_filter(self) -> None:
        app = _make_app()
        with patch("anteroom.routers.search.storage") as mock_storage:
            mock_storage.list_conversations.return_value = [
                {"id": "c1", "title": "Note Conv", "type": "note", "message_count": 3}
            ]
            client = TestClient(app)
            resp = client.get("/api/search?q=hello&mode=keyword&type=note")
            assert resp.status_code == 200
            data = resp.json()
            assert data["mode"] == "keyword"
            mock_storage.list_conversations.assert_called_once()
            call_kwargs = mock_storage.list_conversations.call_args
            assert call_kwargs.kwargs.get("conversation_type") == "note"

    def test_keyword_search_results_include_type(self) -> None:
        app = _make_app()
        with patch("anteroom.routers.search.storage") as mock_storage:
            mock_storage.list_conversations.return_value = [
                {"id": "c1", "title": "A Doc", "type": "document", "message_count": 1}
            ]
            client = TestClient(app)
            resp = client.get("/api/search?q=hello&mode=keyword")
            assert resp.status_code == 200
            data = resp.json()
            assert data["results"][0]["type"] == "document"

    def test_invalid_type_filter_rejected(self) -> None:
        app = _make_app()
        client = TestClient(app)
        resp = client.get("/api/search?q=hello&type=invalid")
        assert resp.status_code == 422

    def test_sql_injection_type_filter_rejected(self) -> None:
        app = _make_app()
        client = TestClient(app)
        resp = client.get("/api/search?q=hello&type=chat' OR '1'='1")
        assert resp.status_code == 422

    def test_no_type_filter_passes_none(self) -> None:
        app = _make_app()
        with patch("anteroom.routers.search.storage") as mock_storage:
            mock_storage.list_conversations.return_value = []
            client = TestClient(app)
            resp = client.get("/api/search?q=hello&mode=keyword")
            assert resp.status_code == 200
            call_kwargs = mock_storage.list_conversations.call_args
            assert call_kwargs.kwargs.get("conversation_type") is None


class TestSemanticSearchTypeInResults:
    """Semantic search results include conversation type."""

    def test_semantic_results_include_type(self) -> None:
        service = AsyncMock()
        service.embed = AsyncMock(return_value=[0.1] * 1536)
        app = _make_app(vec_enabled=True, embedding_service=service)

        with patch("anteroom.routers.search.storage") as mock_storage:
            mock_storage.search_similar_messages.return_value = [
                {"message_id": "m1", "conversation_id": "c1", "content": "Hello", "role": "user", "distance": 0.1}
            ]
            mock_storage.get_conversation.return_value = {"title": "My Note", "type": "note"}

            client = TestClient(app)
            resp = client.get("/api/search/semantic?q=hello")
            assert resp.status_code == 200
            data = resp.json()
            assert data["results"][0]["type"] == "note"


class TestSearchEmptyResults:
    """Edge cases for empty search results."""

    def test_keyword_search_empty_results(self) -> None:
        app = _make_app()
        with patch("anteroom.routers.search.storage") as mock_storage:
            mock_storage.list_conversations.return_value = []
            client = TestClient(app)
            resp = client.get("/api/search?q=nonexistent&mode=keyword")
            assert resp.status_code == 200
            data = resp.json()
            assert data["mode"] == "keyword"
            assert data["results"] == []

    def test_semantic_search_empty_source_results(self) -> None:
        service = AsyncMock()
        service.embed = AsyncMock(return_value=[0.1] * 1536)
        app = _make_app(vec_enabled=True, embedding_service=service)

        with patch("anteroom.routers.search.storage") as mock_storage:
            mock_storage.search_similar_messages.return_value = []
            mock_storage.search_similar_source_chunks.return_value = []
            client = TestClient(app)
            resp = client.get("/api/search/semantic?q=nonexistent")
            assert resp.status_code == 200
            data = resp.json()
            assert data["results"] == []
            assert data["source_results"] == []


class TestAutoModeFallback:
    def test_auto_falls_back_to_keyword_when_embed_returns_none(self) -> None:
        service = AsyncMock()
        service.embed = AsyncMock(return_value=None)
        app = _make_app(vec_enabled=True, embedding_service=service)

        with patch("anteroom.routers.search.storage") as mock_storage:
            mock_storage.list_conversations.return_value = []
            client = TestClient(app)
            resp = client.get("/api/search?q=test&mode=auto")
            assert resp.status_code == 200
            assert resp.json()["mode"] == "keyword"
